X = """
anus - butt
arse - butt
arsehole - butt
ass - butt
ass-hat - idiot
ass-pirate - homosexual
assbag - idiot
assbandit - homosexual
assbanger - homosexual
assbite - idiot
assclown - butt
asscock - idiot
asscracker - butt
asses - butts
assface - butt
assfuck - rear-loving
assfucker - homosexual
assgoblin - homosexual
asshat - butt
asshead - idiot
asshole - jerk
asshopper - homosexual
assjacker - homosexual
asslick - idiot
asslicker - Buttlicker
assmonkey - idiot
assmunch - idiot
assmuncher - butt
assnigger - Racial Slur
asspirate - homosexual
assshit - idiot
assshole - butt
asssucker - idiot
asswad - butt
asswipe - butt
bampot - idiot
bastard - illegitimate child
beaner - Mexican
bitch - female dog
bitchass - idiot
bitches - female dogs
bitchtits - homosexual
bitchy - mean
blow job - sexual act
blowjob - sexual act
bollocks - male genitalia
bollox - male genitalia
boner - erection
brotherfucker - homosexual
bullshit - poop
bumblefuck - homosexual
butt plug - cork
butt-pirate - homosexual
buttfucka - homosexual
buttfucker - homosexual
camel toe - female genitalia
carpetmuncher - homosexual
chinc - Chinese
chink - asian
choad - male genitalia
chode - small penis
clit - female genitals
clitface - idiot
clitfuck - sexual act
clusterfuck - mess up
cock - penis
cockass - Jerk
cockbite - idiot
cockburger - idiot
cockface - idiot
cockfucker - idiot
cockhead - idiot
cockjockey - homosexual
cockknoker - homosexual
cockmaster - homosexual
cockmongler - homosexual
cockmongruel - homosexual
cockmonkey - idiot
cockmuncher - homosexual
cocknose - idiot
cocknugget - idiot
cockshit - idiot
cocksmith - homosexual
cocksmoker - homosexual
cocksucker - homosexual
coochie - female genitalia
coochy - female genitalia
coon - African American
cooter - vagina
cracker - Caucasian
cum - semen
cumbubble - idiot
cumdumpster - prostitute
cumguzzler - homosexual
cumjockey - homosexual
cumslut - dirty girl
cumtart - idiot
cunnie - female genitalia
cunnilingus - sexual act
cunt - vagina
cuntface - idiot
cunthole - female genitalia
cuntlicker - homosexual
cuntrag - idiot
cuntslut - idiot
dago - Italian
damn - darn
deggo - Italian
dick - penis
dickbag - idiot
dickbeaters - Hands
dickface - idiot
dickfuck - idiot
dickfucker - homosexual
dickhead - phallace face
dickhole - male genitalia
dickjuice - semen
dickmilk - sperm
dickmonger - homosexual
dicks - penises
dickslap - sexual act
dicksucker - homosexual
dickwad - idiot
dickweasel - idiot
dickweed - idiot
dickwod - idiot
dike - homosexual
dildo - sexual toy
dipshit - idiot
doochbag - idiot
dookie - poop
douche - female hygene product
douche-fag - idiot
douchebag - female hygene accessory
douchewaffle - homosexual
dumass - idiot
dumb ass - idiot
dumbass - idiot
dumbfuck - idiot
dumbshit - idiot
dumshit - idiot
dyke - homosexual
fag - homosexual
fagbag - homosexual
fagfucker - homosexual
faggit - homosexual
faggot - homosexual
faggotcock - homosexual
fagtard - homosexual idiot
fatass - a fat person
fellatio - sexual act
feltch - sexual act
flamer - homosexual
fuck - fornicate
fuckass - idiot
fuckbag - idiot
fuckboy - idiot
fuckbrain - idiot
fuckbutt - butt
fucked - had intercourse
fucker - fornicator
fuckersucker - idiot
fuckface - idiot
fuckhead - butt
fuckhole - jerk
fuckin - sexual act
fucking - freaking
fucknut - idiot
fucknutt - idiot
fuckoff - go away
fucks - sexual act
fuckstick - male genitalia
fucktard - Moron
fuckup - idiot
fuckwad - idiot
handjob - sexual act
hard on - erection
heeb - Jewish Person
hell - heck
ho - woman
hoe - Woman
homo - homosexual
homodumbshit - idiot
honkey - white person
humping - sexual act
fuckwit - dummy
fuckwitt - idiot
fudgepacker - homosexual
gay - homosexual
gayass - butt
gaybob - homosexual
gaydo - homosexual
gayfuck - homosexual
gayfuckist - homosexual
gaylord - homosexual
gaytard - homosexual
gaywad - homosexual
goddamn - goshdarn
goddamnit - goshdarnit
gooch - female genitalia
gook - Chinese
jackass - idiot
jap - japanesse person
jerk off - masturbate
jigaboo - African American
jizz - Semen
jungle bunny - african american
junglebunny - african american
kike - Jewish Person
kooch - female genitalia
kootch - female genitalia
kunt - female genitalia
kyke - Jewish person
lesbian - homosexual
lesbo - homosexual
lezzie - homosexual
mcfagget - homosexual
mick - irish
minge - female genitalia
mothafucka - Jerk
motherfucker - mother lover
motherfucking - fornicating with mother
muff - female genitalia
muffdiver - homosexual
negro - african american
nigga - african american
nigger - african american
niggers - African Americans
niglet - african american child
nut sack - male genitalia
nutsack - male genitalia
paki - pakistanien
panooch - femail genitalia
pecker - Penis
peckerhead - idiot
penis - male genitalia
penisfucker - homosexual
penispuffer - homosexual
piss - urinate
pissed - urinated
pissed off - angry
pissflaps - female genitalia
polesmoker - homosexual
pollock - polish person
poon - female genitals
poonani - female genitalia
poonany - vagina
poontang - female genitalia
porch monkey - african american
porchmonkey - African American
punanny - female genitalia
punta - female dog
pussies - Female Genitalias
pussy - female reproductive organ
pussylicking - sexual act
puto - idiot
queef - vaginal fart.
queer - homosexual
queerbait - homosexual
queerhole - homosexual
rimjob - dirty sexual act
sand nigger - middle eastern
sandnigger - middle eastern
schlong - male genitalia
scrote - male genitalia
shit - poop
shitass - idiot
shitbag - idiot
shitbagger - idiot
shitbrains - idiot
shitbreath - Bad Breath
shitcunt - idiot
shitdick - idiot
shitface - pooface
shitfaced - Drunk
shithead - jerk
shithole - idiot
shithouse - bathroom
shitspitter - butt
shitstain - poop
shitter - defecator
shittiest - worst
shitting - pooping
shitty - bad
shiz - poop
shiznit - poop
skank - dirty girl
skeet - semen
skullfuck - sexual act
slut - sexually popular woman
slutbag - sexually popular woman
smeg - poop
snatch - female genitalia
spic - mexican
spick - mexican american
splooge - ejaculate
tard - mentally challenged
testicle - male genitalia
thundercunt - idiot
tit - breast
titfuck - sexual act
tits - breasts
tittyfuck - sexual act
twat - female genitals
twatlips - idiot
twats - vaginas
twatwaffle - homosexual
unclefucker - homosexual
va-j-j - female genitalia
vag - femail genitalia
vagina - female genitalia
vjayjay - female genitalia
wank - sexual act
wetback - Mexican
whore - hussy
whorebag - idiot
whoreface - idiot
wop - Italian
"""


import re
from collections import defaultdict
vulgarities = []
vulgarities_by_category = defaultdict(list)
for w, c in filter(None, re.findall('^(.*?)\s*-\s*(.*?)\s*\n', X, re.MULTILINE)):
    vulgarities_by_category[c.lower()].append(w)
    vulgarities.append(w)






"""
Debugging utilities
"""

import re
import sys
import inspect
import traceback

from arsenal.debug import saverr  # registers hook

# new IPython
# import IPython
# debugger = lambda: IPython.InteractiveShell().debugger(force=True)

# try to use IPython's fancy debugger if available


from IPython import embed as ip
#from IPython.frontend.terminal.embed import InteractiveShellEmbed
#_ip = InteractiveShellEmbed(banner1='')


#    def ip():
#        import pylab as pl
#        pl.ion()
#        pl.show()
#        #_ip()
#        embed()

# TODO: look IPython's debugging stuff..
# http://ipython.org/ipython-doc/dev/api/generated/IPython.core.debugger.html
from IPython.core.debugger import Tracer
set_trace = lambda: Tracer()()

    #def set_trace(frame=None):
    #    # get frame
    #    frame = frame or sys._getframe().f_back
    #    # start ipdb with colors
    #    ip = ipapi.get()
    #    d = Pdb(ip.options.colors)
    #    d.set_trace(frame)
    #
    #def pm():
    #    p = Pdb()
    #    p.reset()
    #    p.interaction(None, sys.last_traceback)

"""
    except ImportError:

        # if ipython is unavailable, use interactive console
        from code import InteractiveConsole
        def ip():
            frame = sys._getframe().f_back
            InteractiveConsole(frame).interact()
"""

#from IPython.Debugger import Pdb
from pdb import set_trace, pm, Pdb


def enable_debug_hook():
    "Register pdb's post-mortem debugger as the handler for uncaught exceptions."
    def debug_hook(*args):
        sys.__excepthook__(*args)
        pm()
    sys.excepthook = debug_hook

enable_pm = enable_debug_hook

def dumpobj(o, callables=False, private=False):
    """
    >>> class A(object):
    ...     x = 10
    ...     def __init__(self, y):
    ...         self.y = y
    ...     def span(self):
    ...         pass
    ...     def __repr__(self):
    ...         return 'A(%r, %r)' % (self.x, self.y)
    ...

    >>> dumpobj(A('hello'))
    A(10, 'hello')
    instance of: A
                       x: int
                       y: str

    >>> dumpobj(A('hello'), callables=0, private=True)
    A(10, 'hello')
    instance of: A
                __dict__: dict
                 __doc__: NoneType
              __module__: str
             __weakref__: NoneType
                       x: int
                       y: str
    """
    print repr(o)
    print 'instance of:', type(o).__name__
    for a in dir(o):
        if not callables and callable(getattr(o, a)):
            continue
        if not private and a.startswith('__'):
            continue
        try:
            print '%20s: %s' % (a, type(getattr(o,a)).__name__)
        except:
            pass

def debug(s, *args, **kwargs):
    """
    >>> def foo():
    ...     bar = 'world'
    ...     debug('hello {bar}')
    >>> foo()
    hello world
    """
    c_frame = inspect.getouterframes(inspect.currentframe(), 1)[1][0]
    c_args, c_varargs, c_varkw, c_locals = inspect.getargvalues(c_frame)
    d = dict(c_locals)
    d.update(kwargs)
    print s.format(*args, **d)


debug_expr_fmt = '[DEBUG:%s] %s -> %r'

# borrowed from IPython
def debug_expr(expr):
    """
    Evaluate and print the value of a string representing a python expression
    in the caller's frame.

    Takes an expression, evaluates it in the caller's frame and prints both
    the given expression and the resulting value (as well as a debug mark
    indicating the name of the calling function.  The input must be of a form
    suitable for eval().

    >>> def foo():
    ...     x = 15
    ...     debug_expr('x')
    ...     debug_expr('x**2 + 2*x + 3')
    ...     f = lambda x: x**2
    ...     debug_expr('f(x)')

    >>> foo()
    [DEBUG:foo] x -> 15
    [DEBUG:foo] x**2 + 2*x + 3 -> 258
    [DEBUG:foo] f(x) -> 225
    """
    cf = sys._getframe(1)   # caller frame
    val = eval(expr, cf.f_globals, cf.f_locals)
    print debug_expr_fmt % (cf.f_code.co_name, expr, val)


def debugx(obj):
    """
    I often write debugging print statements which look like
      >>> somevar = 'somevalue'
      >>> print 'somevar:', somevar
      somevar: somevalue

    What this function attempts to do is provide a shortcut
      >>> debugx(somevar)
      somevar: somevalue

    Note: that we do not need to pass strings to this function.
    >>> def foo():
    ...     x = 15
    ...     debugx(x)
    ...     debugx(x**2 + 2*x + 3)
    ...     f = lambda x: x**2
    ...     debugx(f(x))

    >>> foo()
    x: 15
    x**2 + 2*x + 3: 258
    f(x): 225

    Warning: this should only be used for debugging because it relies on
    introspection, which can be really slow and sometimes even buggy.
    """
    cf = sys._getframe(1)
    ctx_lines = inspect.getframeinfo(cf).code_context
    code = ''.join(map(str.strip, ctx_lines))
    code = re.sub('debugx\((.*)\)', r'\1', code)
    print code + ':', obj


# TIMV: would it be possibe to change this function to work without raising
#       and exception?
def framedump():
    """
    Print the usual traceback information, followed by a listing of all the
    local variables in each frame. If this function is called when an exception
    has been thrown the framedump will start at the origin of the exception
    not where it was caught.
    """

    # Move to the frame where the exception occurred, which is often not the
    # same frame where the exception was caught.
    tb = sys.exc_info()[2]
    if tb is not None:
        while 1:
            if not tb.tb_next:
                break
            tb = tb.tb_next
        f = tb.tb_frame
    else:                             # no exception occurred
        f = sys._getframe()

    # get the stack frames
    stack = []
    while f:
        stack.append(f)
        f = f.f_back
    stack.reverse()

    print 'Traceback:'
    print '=========='
    print traceback.format_exc()

    print 'Locals by frame:'
    print '================'
    for frame in stack:
        print 'Frame %s in %s at line %s' % (frame.f_code.co_name,
                                             frame.f_code.co_filename,
                                             frame.f_lineno)
        for key, value in frame.f_locals.iteritems():
            print '%20s = %r' % (key, value)

        print
        print


if __name__ == '__main__':
    import doctest

    def example():
        """
        A simple example where this approach comes in handy.

        Basically, we have a simple function which manipulates all the
        strings in a list. The function doesn't do any error checking, so when
        we pass a list which contains something other than strings, we get an
        error.
        """

        data = ["1", "2", 3, "4"]  # Typo: We 'forget' the quotes on data[2]
        def pad4(seq):
            """Pad each string in seq with zeros, to four places."""
            return_value = []
            for thing in seq:
                return_value.append("0" * (4 - len(thing)) + thing)
            return return_value

        print '============================================================'
        print 'The usual information'
        print '============================================================'
        # First, show the information we get from a normal traceback.print_exc().
        try:
            pad4(data)
        except:
            traceback.print_exc()

        print
        print '============================================================'
        print 'Tracebacks with the frame dump'
        print '============================================================'

        # With our new function it is to see the bad data that
        # caused the problem. The variable 'thing' has the value 3, so we know
        # that the TypeError we got was because of that. A quick look at the
        # value for 'data' shows us we simply forgot the quotes on that item.

        if 1:
            try:
                pad4(data)
            except:
                framedump()
        else:
#            pad4(data)
            framedump()

    example()

    import doctest
    doctest.testmod()



## from contextlib import contextmanager
## from functools import wraps
##
## @contextmanager
## def assert_throws_ctx(*exc):
##     passed = False
##     got = None
##     try:
##         yield
##     except exc:
##         passed = True
##     except Exception as e:
##         got = e
##     else:
##         # since None isn't realy an exception, we have to special case it.
##         if None in exc:
##             passed = True
##     finally:
##         if not passed:
##             msg = ' or '.join(e.__name__ if e is not None else 'None' for e in exc)
##             raise AssertionError('did not raise required %s. Got %s instead.' % (msg, got))
##
## def assert_throws_dec(exc):
##     def wrap(f):
##         @wraps(f)
##         def wrap2(*args,**kw):
##             with assert_throws_ctx(exc):
##                 return f(*args,**kw)
##         return wrap2
##     return wrap


from arsenal.recipes.contextdecorator import ContextDecorator

class assert_throws(ContextDecorator):
    """
    Contextmanager + Decorator for asserting that a certain exceptions (or no
    exception) will arise with it's context.

    As a context manager:

      >>> with assert_throws(ZeroDivisionError):
      ...     1/0

      >>> with assert_throws(None):
      ...     pass

      >>> with assert_throws(None, ZeroDivisionError):
      ...     pass

      >>> with assert_throws(ZeroDivisionError):
      ...     pass
      Traceback (most recent call last):
          ...
      AssertionError: did not raise required ZeroDivisionError. Got None instead.

      >>> with assert_throws(AssertionError, ZeroDivisionError):
      ...     pass
      Traceback (most recent call last):
          ...
      AssertionError: did not raise required AssertionError or ZeroDivisionError. Got None instead.

    As a decorator:

      >>> @assert_throws(ZeroDivisionError)
      ... def foo():
      ...     1 + 1   # should not raise ZeroDivisionError
      ...
      >>> foo()
      Traceback (most recent call last):
          ...
      AssertionError: did not raise required ZeroDivisionError. Got None instead.
    """

    def __init__(self, *expect):
        self.expect = expect
        super(assert_throws, self).__init__()

    def after(self, exc, val, _):
        msg = ' or '.join('None' if e is None else e.__name__ for e in self.expect)

        if exc is None:
            if None not in self.expect:
                raise AssertionError('did not raise required %s. Got %s instead.' % (msg, None))
        else:
            if not issubclass(exc, self.expect):
                raise AssertionError('did not raise required %s. Got %s instead.' % (msg, exc.__name__))

        return True


if __name__ == '__main__':
    import doctest
    doctest.testmod()

    def test():

        @assert_throws(ZeroDivisionError)
        def test_assert_throws1():
            return 1/0
        test_assert_throws1()

        @assert_throws(None)
        def test_assert_throws2():
            return 2 + 2
        test_assert_throws2()

        with assert_throws(Exception):
            print 1/0

        with assert_throws(ZeroDivisionError):
            print 1/0

        with assert_throws(None):
            pass

        try:
            with assert_throws(ZeroDivisionError, TypeError, ValueError):
                pass
        except AssertionError:
            pass
        else:
            raise AssertionError('test failed.')

        print 'Passed basic tests.'

    test()



from unitgroup import UnitGroup
import unitdata, option

UNKNOWN_UNIT         = 'UNKNOWN UNIT'
CONFORMABILITY_ERROR = 'CONFORMABILITY ERROR'

options = option.Option('convertall', 20)
#options.loadAll(["DecimalPlaces 8", "SciNotation yes", "FixedDecimals no"])

data = unitdata.UnitData()
data.readData()

def convert(value, from_unit, to_unit):

    fromUnit = UnitGroup(data, options)
    fromUnit.update(from_unit)
    fromUnit.reduceGroup()
    
    toUnit = UnitGroup(data, options)
    toUnit.update(to_unit)
    toUnit.reduceGroup()

    if not fromUnit.reducedList or not toUnit.reducedList:
        return UNKNOWN_UNIT

    if not fromUnit.categoryMatch(toUnit):
        return CONFORMABILITY_ERROR
    else:
        num = float(value)
        return fromUnit.convert(num, toUnit)



#!/usr/bin/env python
"""
Find most informative features ranked by information gain (i.e. the ID3 hueristic for decision trees).

Input: a tab-delimited file where each line starts with a label followed by features.
Output: a sorted list of the most informative features.
"""

import sys
from numpy import zeros, fromiter, int32
from arsenal.math import kl_divergence, normalize, lidstone
from arsenal.alphabet import Alphabet
from collections import defaultdict


def read_tab_file(f):
    """
    Load really simple tab-separated file format for labeled data. Label is the
    first column, features are the remaining columns.
    """
    for line in f:
        line = line.strip()
        line = line.split('\t')
        if len(line) <= 1:
            continue
        yield line[0], line[1:]


def integerize(data):
    """
    Integerize dataset
    returns a triple (label alphabet, feature alphabet, integerized dataset)
    """
    F = Alphabet()
    L = Alphabet()
    I = [(L[label], fromiter(F.map(features), dtype=int32)) for label, features in data]
    return (L, F, I)



def kl_filter(data,
              verbose=True,
              progress=False,
              out=sys.stdout,
              feature_label_cuttoff=0,
              feature_count_cuttoff=0,
              do_label_count=False):
    """
    data = (label, [features ...])

    KL is a synonym for Information Gain

    KL( p(label) || p(label|feature) )
    """
    (L, F, data) = integerize(data)

    if do_label_count:
        label_count = defaultdict(int)
        for label, features in data:
            label_count[label] += 1
        label_count = label_count.items()
        label_count.sort(key=lambda x: -x[1])  # sort by count
        print 'label count'
        for k,v in label_count:
            print '%20s => %s' % (k, v)
        sys.exit(0)

    K = len(L)
    M = len(F)

    if progress:
        from arsenal.iterextras import iterview
    else:
        iterview = lambda x, *a, **kw: x

    if progress:
        print >> sys.stderr, '\nTally'

    # label-feature tally (note: we ignore dulicate features)
    counts = zeros((K,M))
    for y, fv in iterview(data, every=5000):
        counts[y, fv] += 1

    feature_counts = counts.sum(axis=0)

    if feature_count_cuttoff > 0:
        cut = feature_counts < feature_count_cuttoff

        #if verbose:
        print >> sys.stderr, '%s of %s below cutoff of %s' \
            % (cut.sum(), len(feature_counts), feature_count_cuttoff)

        if progress:
            print >> sys.stderr, '%s / %s (%.2f%%) features below cuttoff' % \
                (cut.sum(), M, cut.sum()*100.0/M)

        # zero-out features below cuttoff
        counts[:, cut] = 0

    if feature_label_cuttoff:
        cut = counts < feature_label_cuttoff

        if progress:
            print >> sys.stderr, '%s / %s (%.2f%%) feature-label pairs below cuttoff' % \
                (cut.sum(), K*M, cut.sum()*100.0/(K*M))

        # zero-out features below cuffoff
        counts[cut] = 0

    label_prior = normalize(counts.sum(axis=1))

    # compute KL
    if progress:
        print >> sys.stderr, '\nKL'

    KL = zeros(M)
    for f in iterview(xrange(M), every=5000):
        label_given_f = lidstone(counts[:,f], 0.0001)   # avoids divide-by-zero
        KL[f] = -kl_divergence(label_prior, label_given_f)

    # print KL-feature, most-informative first
    for i in KL.argsort():

        z = counts[:,i].sum()

        if z == 0:
            continue

        p = counts[:,i] * 1.0 / z

        l = [(v, k) for k,v in zip(L, p) if v > 0]
        l.sort()

        z = (-KL[i], F.lookup(i), l)

        if verbose:
            print >> out, '%8.6f\t%s\t%s' % (-KL[i], int(counts[:,i].sum()), F.lookup(i)), '\t\033[32m', ' '.join('%s(%s)' % (k,v) for v, k in l), '\033[0m'

        yield z


if __name__ == '__main__':
    list(kl_filter(read_tab_file(file(sys.argv[1]) if len(sys.argv) == 2 else sys.stdin)))



import numpy
import scipy
import scipy.linalg

try:
    from arsenal.iterextras import iterview
except ImportError:
    iterview = lambda x, *args, **kw: x

def ransac(data, model, n, k, t, d, debug=False):
    """
    fit model parameters to data using the RANSAC algorithm

    This implementation written from pseudocode found at
        http://en.wikipedia.org/w/index.php?title=RANSAC&oldid=116358182

    Given:
        data - a set of observed data points
        model - a model that can be fitted to data points
        n - the minimum number of data values required to fit the model
        k - the maximum number of iterations allowed in the algorithm
        t - a threshold value for determining when a data point fits a model
        d - the number of close data values required to assert that a model fits well to data
    Return:
        bestfit - model parameters which best fit the data (or None if no good model is found)
    """

    bestfit          = None
    besterr          = numpy.inf
    best_inlier_idxs = None

    for i in iterview(xrange(k), 250):
        # randomly partition data (random_partition returns two arrays of ids)
        maybe_idxs, test_idxs = random_partition(n, data.shape[0])

        # get data points for each id
        maybeinliers = data[maybe_idxs,:]
        test_points  = data[test_idxs]

        # fit model and check error on the test points
        maybemodel   = model.fit(maybeinliers)
        test_err     = model.get_error(test_points, maybemodel)

        # pick indices of test_points with acceptable error (below threshold)
        also_idxs    = test_idxs[test_err < t]
        alsoinliers  = data[also_idxs,:]

        if debug:
            print 'test_err.min()', test_err.min()
            print 'test_err.max()', test_err.max()
            print 'numpy.mean(test_err)', numpy.mean(test_err)
            print 'iteration %d: len(alsoinliers) = %d' % (i, len(alsoinliers))

        # Do we have enough values not included in the fit-partition to assert that
        # maybemodel fits well-enough?
        if len(alsoinliers) > d:

            betterdata  = numpy.concatenate((maybeinliers, alsoinliers))
            bettermodel = model.fit(betterdata)
            better_errs = model.get_error(betterdata, bettermodel)  # SSE per row
            thiserr     = numpy.mean(better_errs)

            # only keep the best model, error, and data
            if thiserr < besterr:
                bestfit = bettermodel
                besterr = thiserr
                best_inlier_idxs = numpy.concatenate((maybe_idxs, also_idxs))

    if bestfit is None:
        raise ValueError("did not meet fit acceptance criteria")

    return bestfit, {'inliers': best_inlier_idxs}

def random_partition(n, n_data):
    """return n random rows of data (and also the other len(data)-n rows)"""
    all_idxs = numpy.arange(n_data)
    numpy.random.shuffle(all_idxs)
    idxs1 = all_idxs[:n]
    idxs2 = all_idxs[n:]
    return idxs1, idxs2

# TODO: model and params seems to be used interchangeably in some parts..
class LinearLeastSquaresModel(object):
    """
    linear system solved using linear least squares

    This class serves as an example that fulfills the model interface
    needed by the ransac() function.
    """
    def __init__(self, input_columns, output_columns, debug=False):
        self.input_columns = input_columns
        self.output_columns = output_columns
        self.debug = debug
    def fit(self, data):
        A = numpy.vstack([data[:,i] for i in self.input_columns]).T
        B = numpy.vstack([data[:,i] for i in self.output_columns]).T
        x, resids, rank, s = scipy.linalg.lstsq(A, B)
        return x
    def get_error(self, data, model):
        A = numpy.vstack([data[:,i] for i in self.input_columns]).T
        B = numpy.vstack([data[:,i] for i in self.output_columns]).T
        B_fit = scipy.dot(A, model)
        err_per_point = numpy.sum((B-B_fit)**2, axis=1) # sum squared error per row
        return err_per_point


from sklearn.linear_model import Ridge

class RidgeRegressionModel(LinearLeastSquaresModel):
    def __init__(self, input_columns, output_columns, debug=False):

        self.alpha = 0.0000000001
        self.m = Ridge(alpha=self.alpha)

        super(RidgeRegressionModel, self).__init__(input_columns, output_columns, debug=debug)

    def fit(self, data):
        A = numpy.vstack([data[:,i] for i in self.input_columns]).T
        B = numpy.vstack([data[:,i] for i in self.output_columns]).T

        self.m.fit(A, B)

        return self.m.coef_   #m.intercept_

    def get_error(self, data, model):
        A = numpy.vstack([data[:,i] for i in self.input_columns]).T
        B = numpy.vstack([data[:,i] for i in self.output_columns]).T
        B_fit = scipy.dot(A, model)
        err_per_point = numpy.sum((B-B_fit)**2, axis=1) # sum squared error per row

        norm = numpy.sqrt(model*model)
        assert norm.shape == (1,1)
        regularizer = 1.0*norm[0,0]

        return err_per_point - regularizer


class Perceptron(LinearLeastSquaresModel):
    def __init__(self, input_columns, output_columns, debug=False):
        LinearLeastSquaresModel.__init__(self, input_columns, output_columns, debug=debug)

    def fit(self, data):
        A = numpy.vstack([data[:,i] for i in self.input_columns]).T
        B = numpy.vstack([data[:,i] for i in self.output_columns]).T
        w = numpy.zeros((1, len(A[0]) + 1))
        for i in xrange(100):
            for x, y in zip(A,B):
                x = numpy.append(x, 1)
                z = numpy.dot(w, x)
                w += 0.000005*((y-z)*x - 100.0*w)
        return w


def test():
    # generate perfect input data
    n_samples = 300
    n_inputs  = 1
    n_outputs = 1
    A_exact     = 20*numpy.random.random((n_samples, n_inputs))
    perfect_fit = 60*numpy.random.normal(size=(n_inputs, n_outputs)) # the model
    B_exact     = scipy.dot(A_exact, perfect_fit)
    assert B_exact.shape == (n_samples, n_outputs)

    # add a little Gaussian noise (linear least squares alone should handle this well)
    A_noisy = A_exact + numpy.random.normal(size=A_exact.shape)
    B_noisy = B_exact + numpy.random.normal(size=B_exact.shape)

    # add some outliers, by perturbing a portion of the data
    if 1:
        n_outliers = 40
        outlier_idxs, non_outlier_idxs = random_partition(n_outliers, A_noisy.shape[0])
        A_noisy[outlier_idxs] = 20*numpy.random.random((n_outliers, n_inputs)) + 20
        B_noisy[outlier_idxs] = 50*numpy.random.normal(size=(n_outliers, n_outputs))

    # setup model
    all_data       = numpy.hstack((A_noisy, B_noisy))
    input_columns  = range(n_inputs)
    output_columns = [n_inputs+i for i in xrange(n_outputs)]
    debug = False

    least_squares = LinearLeastSquaresModel(input_columns, output_columns, debug=debug)
    perceptron = Perceptron(input_columns, output_columns, debug=debug)
    ridge_model = RidgeRegressionModel(input_columns, output_columns, debug=debug)

    #model = perceptron
    model = least_squares
    #model = ridge_model

    # run RANSAC algorithm
    ransac_fit, ransac_data = ransac(all_data, model, 30, 50, 1e3, 10, debug=debug)

    # what happens if we fit the model to all the data?
    linear_fit = least_squares.fit(all_data)
    ridge_fit = ridge_model.fit(all_data)
    perceptron_fit = perceptron.fit(all_data)

    # plotting
    import pylab
    sort_idxs = numpy.argsort(A_exact[:,0])
    A_col0_sorted = A_exact[sort_idxs]       # maintain as rank-2 array

    # color the outliers generated differently from all data?
    if 1:
        pylab.plot(A_noisy[non_outlier_idxs,0], B_noisy[non_outlier_idxs,0], 'k.', label='noisy data' )
        pylab.plot(A_noisy[outlier_idxs,0],     B_noisy[outlier_idxs,0],     'r.', label='outlier data' )
    else:
        pylab.plot(A_noisy[:,0], B_noisy[:,0], 'k.', label='data')

    # show data point used in RANSAC's fit
    if 1:
        pylab.plot(A_noisy[ransac_data['inliers'],0], B_noisy[ransac_data['inliers'],0], 'bx', label='RANSAC data')

    pylab.plot(A_col0_sorted[:,0], numpy.dot(A_col0_sorted, ransac_fit)[:,0],  label='RANSAC fit')
    pylab.plot(A_col0_sorted[:,0], numpy.dot(A_col0_sorted, perfect_fit)[:,0], label='exact system')
    pylab.plot(A_col0_sorted[:,0], numpy.dot(A_col0_sorted, linear_fit)[:,0],  label='linear fit')
    pylab.plot(A_col0_sorted[:,0], numpy.dot(A_col0_sorted, ridge_fit)[:,0],  label='ridge fit')
    pylab.plot(A_col0_sorted[:,0], numpy.dot(A_col0_sorted, perceptron_fit)[:,0],  label='perceptron fit')

    pylab.legend()

    #pylab.legend()
    pylab.show()


if __name__=='__main__':
    test()



import re

from arsenal.nlp import gazetteers

class pattern(object):
    """
    Utility for common regex things...

    Note: regular expressions are always compiled with the VERBOSE flag.
    """
    def __init__(self, p, handler=lambda *x: bool(x or True), flags=re.IGNORECASE):
        self.re_matches  = re.compile('^%s$' % p, re.VERBOSE|flags)
        self.re_contains = re.compile('.*?%s.*?' % p, re.VERBOSE|flags)
        self._handler = handler

    def handler(self, *args):
        # need to strip off self argument
        return self._handler(*args)

    def contains(self, x):
        if hasattr(x, 'form'): x = x.form
        m = self.re_contains.match(x)
        if m: return self.handler(*m.groups())

    def matches(self, x):
        if hasattr(x, 'form'): x = x.form
        m = self.re_matches.match(x)
        if m: return self.handler(*m.groups())

    __call__ = matches


## maybe i want to distinguish between sometimes and always ordinals
ordinal = pattern("""
    (?:
        \d+(?:st|nd|rd|th)
        |first|second|third|fourth|fifth|sixth|seventh|eighth|ninth|tenth
        |eleventh|twelfth|thirteenth|fourteenth|fifteenth|sixteenth
        |seventeenth|eighteenth|nineteenth
        |twentieth|thirtieth|fou?rtieth|fiftieth|sixtieth|seventieth
        |eightieth|ninetieth
        |hundredth|thousandth|millionth|billionth
    )
    """)

fraction_denom = pattern("""
    (?:
        half|halve|third|fourth|fifth|sixth|seventh|eighth|ninth|tenth
        |eleventh|twelfth|thirteenth|fourteenth|fifteenth|sixteenth
        |seventeenth|eighteenth|nineteenth
        |twentieth|thirtieth|fou?rtieth|fiftieth|sixtieth|seventieth
        |eightieth|ninetieth
        |hundredth|thousandth|millionth|billionth
    )s   # <-- NOTE the 's'
    """)

written_number = pattern('|'.join(gazetteers.numbers))

digits = pattern('(\d+)')
four_digits = pattern('(\d\d\d\d)')
two_digits = pattern('(\d\d)')

two_letter = pattern('[A-Z][A-Z]')
initial = pattern('[A-Z]\.')
abbrev = pattern('([A-Z]?[a-z]+\.)')
punct = pattern('[!"#&\'\(\)/:;<>\?@\[\]\_`{\|}~^]+')
alpha = pattern('[A-Za-z]+')
roman = pattern("""
    (M?M?M?(?:CM|CD|D?C?C?C?)(?:XC|XL|L?X?X?X?)(?:IX|IV|V?II?|III))
    """, lambda m: m != 'I')   # don't match 'I'

numeric = pattern('((?:\d{1,3}(?:\,\d{3})*|\d+)(?:\.\d+)?)')

doftw = pattern("""
    (?: Mon
        |Tues?
        |Wed(?:nes)?
        |Thurs?
        |Fri
        |Satu?r?
        |Sun
    )(?:day|\.)
    """)

month = pattern("""
    (?: Jan(?:uary|\.)
        |Febr?(?:uary|\.)
        |Mar(?:ch|\.)
        |Apr(?:il|\.)
        |May
        |Jun(?:e|\.)
        |Jul(?:y|\.)
        |Aug(?:ust|\.)
        |Sept?(?:ember|\.)
        |Oct(?:ober|\.)
        |Nov(?:ember|\.)
        |Dec(?:ember|\.)
    )
    """)

day_words = pattern("""
    (?: today
        |tomorrow
        |yesterday
        |morning
        |afternoon
        |evening
    )""")

possible_year = pattern("""
    (?: \d\d\d\d (?:\s*s)? | \'? \d\d (?:\s*?s)? )
    """)

def validate_time(hr, colon, m, ampm, tzone):
    if ampm in ('a','p','A','P') and not colon:
        return False
    if not (0 <= int(m or 0) <= 59):
        return None
    if ampm:
        if not (1 <= int(hr) <= 12):
            return None
    else:
        if not (0 <= int(hr) <= 24) or m is None:
            return None
    if not (colon or tzone or ampm):
        return None
    return True

time = pattern("""(\d\d?) \s*? (?: (\:)? \s*? (\d\d))? \s* ([ap]\.m\.?|[ap]m|[ap])?
\s* (?: \(? (GMT|EST|PST|CST)? \)? )? (?:\W|$)""", handler=validate_time)

def capitalized(tk):
    return tk.form[0].isupper() and tk.form[1:].islower()



"""
Disk And Execution MONitor (Daemon)

Configurable daemon behaviors:

  1.) The current working directory set to the "/" directory.
  2.) The current file creation mode mask set to 0.
  3.) Close all open files.
  4.) Redirect standard I/O streams to "/dev/null".

Based on an implementation by Chad J. Schroeder
"""

import os
import sys
import resource

# Notes
# ========
# os._exit() vs. os.exit()
#   _exit, unlike exit(), does not call any atexit hooks or signal handlers. 
#   Using exit() may cause all stdio streams to be flushed twice and any
#   temporary files may be unexpectedly removed. 

def daemonize(workdir='/', log=os.devnull, pidfile=None, umask=0):
    """
    Detach a process from the controlling terminal and run it in the
    background as a daemon.

    redirect_to:
      Redirect standard I/O file descriptors to the specified file
      (by default, /dev/null).

      Since the daemon has no controlling terminal, most daemons redirect 
      stdin, stdout, and stderr to /dev/null. This is done to prevent
      side-effects from reads and writes to the standard I/O file descriptors.

      Passing workdir=None will leave the process in it's current directory

    workdir:
      Default working directory for the daemon.

      Since the current working directory may be a mounted filesystem, we
      avoid the issue of not being able to unmount the filesystem at
      shutdown time by changing it to the root directory (by default).

    umask:
      File mode creation mask of the daemon.    

    Note:
      * This function will close all open file descriptors. If you processes
        reads/writes to any files, it must call daemonize before opening them.
    """

    # flush everything before daemoizing (timv: this might not be necessary)
    sys.stdout.flush()
    sys.stderr.flush()

    # Fork a child process so the parent can exit.  
    # * returns control to the command-line or shell. 
    # * guarantees child will not be a process group leader, since the child 
    #   receives a new process ID and inherits the parent's process group ID.
    # * required to insure that the next call to os.setsid is successful.
    pid = os.fork()

    if pid == 0:	# The first child.
        # To become the session leader of this new session and the process group
        # leader of the new process group, we call os.setsid().  The process is
        # also guaranteed not to have a controlling terminal.
        os.setsid()

        # Fork a second child
        # ===================
        # Fork a second child and exit immediately to prevent zombies. This
        # causes the second child process to be orphaned, making the init
        # process responsible for its cleanup.  And, since the first child is
        # a session leader without a controlling terminal, it's possible for
        # it to acquire one by opening a terminal in the future (System V-
        # based systems).  This second fork guarantees that the child is no
        # longer a session leader, preventing the daemon from ever acquiring
        # a controlling terminal.
        pid = os.fork()

        if pid == 0:	# The second child.

            # Since the current working directory may be a mounted filesystem, we
            # avoid the issue of not being able to unmount the filesystem at
            # shutdown time by changing it to the root directory.
            if workdir is not None:
                os.chdir(workdir)

            # We probably don't want the file mode creation mask inherited from
            # the parent, so we give the child complete control over permissions.
            os.umask(umask)

        else:
            # Exit parent (the first child) of the second child.
            os._exit(0)
    else:
        os._exit(0)	# Exit parent of the first child.


    # Close all open file descriptors.  This prevents the child from keeping
    # open any file descriptors inherited from the parent.
    maxfd = resource.getrlimit(resource.RLIMIT_NOFILE)[1]
    if maxfd == resource.RLIM_INFINITY:
        # Default maximum for the number of available file descriptors.
        maxfd = 1024

    # close all file descriptors.
    os.closerange(0, maxfd)

    # Redirect the standard I/O file descriptors to the specified file.  Since
    # the daemon has no controlling terminal, most daemons redirect stdin,
    # stdout, and stderr to /dev/null.  This is done to prevent side-effects
    # from reads and writes to the standard I/O file descriptors.

    # This call to open is guaranteed to return the lowest file descriptor,
    # which will be 0 (stdin), since it was closed above.
    fd = os.open(log, os.O_RDWR|os.O_CREAT)  # standard input (0)

    # Duplicate fd to stdout and stderr
    os.dup2(fd, 1)    # standard output (1)
    os.dup2(fd, 2)    # standard error (2)

    if pidfile:
        with file(pidfile, 'wb') as f:
            f.write(str(os.getpid()))


if __name__ == "__main__":
    def test():
        pid = os.getpid()
        print 'pid:', pid
        daemonize(None, log='daemon.out', pidfile='daemon.PID')
        print "Hello I'm a daemon (pid=%s) and this is my standard out." % pid
        print 'I am now sleeping for 20 seconds.'
        import time
        time.sleep(20)
        print 'I am awake! ready or not, here I come!'

    daemonize(None)
    os.system(' '.join(sys.argv[1:]))



class OrderedSet(object):
    """
    Set which remembers insertion ordering allowed iteration while changing size
    and determinism.
    """
    __slots__ = 'set', 'list'
    def __init__(self):
        self.set = set()
        self.list = []
    def __contains__(self, item):
        return item in self.set
    def __iter__(self):
        return iter(self.list)
    def add(self, item):
        if item not in self.set:
            self.set.add(item)
            self.list.append(item)



import weakref


class ondemand(property):
    """A property that is loaded once from a function."""
    def __init__(self, fget, doc=None):
        property.__init__(self, fget=self.get, fdel=self.delete, doc=doc)
        self.loadfunc = fget
        self.values = weakref.WeakKeyDictionary()
    def get(self, obj):
        if obj not in self.values:
            self.load(obj)
        return self.values[obj]
    def load(self, obj):
        self.values[obj] = self.loadfunc(obj)
    def delete(self, obj):
        try:
            del self.values[obj]
        except:
            pass


from arsenal.cache.lazy import lazy
from arsenal.misc import deprecated
@deprecated(lazy)
def cachedproperty(*args, **kw):
    return lazy(*args, **kw)


'''
class cachedproperty(object):
    """
    Lazy-loading read/write property descriptor.
    Value is stored locally in descriptor object. If value is not set when
    accessed, value is computed using given function. Value can be cleared
    by calling 'del'.
    """
    def __init__(self, func):
        self._func = func
        self._values = {}
        self.__name__ = func.__name__
        self.__doc__ = func.__doc__

    def __get__(self, obj, obj_class):
        if obj is None:
            return obj
        if obj not in self._values or self._values[obj] is None:
            self._values[obj] = self._func(obj)
        return self._values[obj]

    def __set__(self, obj, value):
        self._values[obj] = value

    def __delete__(self, obj):
        if self.__name__ in obj.__dict__:
            del obj.__dict__[self.__name__]
        self._values[obj] = None
'''


if __name__ == '__main__':
    from collections import defaultdict

    class Foo(object):
        def __init__(self, x):
            self.x = x
            self.log = defaultdict(int)       # track the number of calls to our cached properties/methods
        @ondemand
        def my_ondemand(self):
            self.log['ondemand'] += 1
            return 'ON DEMAND'.split()        # return something mutable
        @cachedproperty
        def my_cached(self):
            self.log['cachedproperty'] += 1
            return 'CACHED PROPERTY'.split()
        @lazy
        def my_lazy(self):
            self.log['my_lazy'] += 1
            return 'LAZY PROPERTY'.split()
        @lazy
        def my_lazy_list(self):
            self.log['my_lazy_list'] += 1
            for i in xrange(10):
                yield i

    def test():
        import pickle

        def checks(x):
            o = x.my_ondemand
            c = x.my_cached
            l = x.my_lazy
            assert x.my_ondemand is o
            assert x.my_cached is c
            assert x.my_lazy is l
            assert min(x.log.values()) == max(x.log.values()) == 1

            ll = x.my_lazy_list
            assert ll == range(10), ll
            assert ll is x.my_lazy_list

            print x.log
            assert all(v == 1 for v in x.log.values())

        foo = Foo('XXX')
        checks(foo)
        foo.log.clear()

        print 'pickling and unpickling..'
        foo2 = pickle.loads(pickle.dumps(foo))

        # should still call both lazy properties again.
        checks(foo2)

    test()



#!/usr/bin/env python

#****************************************************************************
# option.py, provides classes to read and set user preferences
#
# Copyright (C) 2004, Douglas W. Bell
#
# This is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License, Version 2.  This program is
# distributed in the hope that it will be useful, but WITTHOUT ANY WARRANTY.
#*****************************************************************************

import sys, os.path, codecs

class Option:
    """Stores and retrieves string options"""
    def __init__(self, fileName, keySpaces=20):
        self.path = ''
        if fileName:
            self.path = os.environ.get('HOME', '')
            if os.path.exists(self.path) and sys.platform != 'win32':
                self.path = os.path.join(self.path, '.' + fileName)
            else:
                self.path = os.path.join(os.path.abspath(sys.path[0]), \
                                         '%s.ini' % fileName)
        self.keySpaces = keySpaces
        self.dfltDict = {}
        self.userDict = {}
        self.dictList = (self.userDict, self.dfltDict)
        self.chgList = []

    def loadAll(self, defaultList):
        """Reads defaultList & file, writes file if required
           return true if file read"""
        self.loadSet(defaultList, self.dfltDict)
        if self.path:
            try:
                f = codecs.open(self.path, 'r', 'utf-8')
            except IOError:
                try:
                    f = codecs.open(self.path, 'w', 'utf-8')
                except IOError:
                    print 'Error - could not write to config file', self.path
                else:
                    f.writelines([line + '\n' for line in defaultList])
                    f.close()
            else:
                self.loadSet(f.readlines(), self.userDict)
                f.close()
                return 1
        return 0

    def loadSet(self, list, data):
        """Reads settings from list into dict"""
        for line in list:
            line = line.split('#', 1)[0].strip()
            if line:
                item = line.split(None, 1) + ['']   # add value if blank
                data[item[0]] = item[1].strip()

    def addData(self, key, strData, storeChange=0):
        """Add new entry, add to write list if storeChange"""
        self.userDict[key] = strData
        if storeChange:
            self.chgList.append(key)

    def boolData(self, key):
        """Returns true or false from yes or no in option data"""
        for data in self.dictList:
            val = data.get(key)
            if val and val[0] in ('y', 'Y'):
                return 1
            if val and val[0] in ('n', 'N'):
                return 0
        print 'Option error - bool key', key, 'is not valid'
        return 0

    def numData(self, key, min=None, max=None):
        """Return float from option data"""
        for data in self.dictList:
            val = data.get(key)
            if val:
                try:
                    num = float(val)
                    if (min == None or num >= min) and \
                       (max == None or num <= max):
                        return num
                except ValueError:
                    pass
        print 'Option error - float key', key, 'is not valid'
        return 0

    def intData(self, key, min=None, max=None):
        """Return int from option data"""
        for data in self.dictList:
            val = data.get(key)
            if val:
                try:
                    num = int(val)
                    if (min == None or num >= min) and \
                       (max == None or num <= max):
                        return num
                except ValueError:
                    pass
        print 'Option error - int key', key, 'is not valid'
        return 0

    def strData(self, key, emptyOk=0):
        """Return string from option data"""
        for data in self.dictList:
            val = data.get(key)
            if val != None:
                if val or emptyOk:
                    return val
        print 'Option error - string key', key, 'is not valid'
        return ''

    def changeData(self, key, strData, storeChange):
        """Change entry, add to write list if storeChange
           Return true if changed"""
        for data in self.dictList:
            val = data.get(key)
            if val != None:
                if strData == val:  # no change reqd
                    return 0
                self.userDict[key] = strData
                if storeChange:
                    self.chgList.append(key)
                return 1
        print 'Option error - key', key, 'is not valid'
        return 0

    def writeChanges(self):
        """Write any stored changes to the option file - rtn true on success"""
        if self.path and self.chgList:
            try:
                f = codecs.open(self.path, 'r', 'utf-8')
                fileList = f.readlines()
                f.close()
                for key in self.chgList[:]:
                    hitList = [line for line in fileList if \
                               line.strip().split(None, 1)[:1] == [key]]
                    if not hitList:
                        hitList = [line for line in fileList if \
                                   line.replace('#', ' ', 1).strip().\
                                   split(None, 1)[:1] == [key]]
                    if hitList:
                        fileList[fileList.index(hitList[-1])] = '%s%s\n' % \
                                (key.ljust(self.keySpaces), self.userDict[key])
                        self.chgList.remove(key)
                for key in self.chgList:
                    fileList.append('%s%s\n' % (key.ljust(self.keySpaces), \
                                                self.userDict[key]))
                f = codecs.open(self.path, 'w', 'utf-8')
                f.writelines([line for line in fileList])
                f.close()
                return 1
            except IOError:
                print 'Error - could not write to config file', self.path
        return 0



from numpy import zeros

def default_cost(a,b):
    if a == b:
        return 0.0
    elif a is None or b is None:
        return 1.0
    else:
        return 1000.0

def lcs(a, b, cost=default_cost):

    N = len(a)
    M = len(b)

    c = zeros((N + 1, M + 1))

    for i in reversed(xrange(N)):
        c[i, M] = cost(a[i], None) + c[i + 1, M]

    for j in reversed(xrange(M)):
        c[N, j] = cost(None, b[j]) + c[N, j + 1]

    for i in reversed(xrange(N)):
        for j in reversed(xrange(M)):
            c[i, j] = min(cost(a[i], b[j]) + c[i + 1, j + 1],
                          cost(a[i], None) + c[i + 1, j],
                          cost(None, b[j]) + c[i, j + 1])

    y = []; i = 0; j = 0
    while i < N or j < M:
        cij = c[i, j]
        if i < N and j < M and cij == cost(a[i], b[j]) + c[i + 1, j + 1]:
            y.append((a[i], b[j]))
            i += 1
            j += 1
        elif i < N and cij == cost(a[i], None) + c[i + 1, j]:
            y.append((a[i], None))
            i += 1
        else:
            assert j < M and cij == cost(None, b[j]) + c[i, j + 1]
            y.append((None, b[j]))
            j += 1

    assert i == N and j == M

    return (c[0,0], y)


def longest_increasing_subsequence(a, verbose=False):

    s = a[:]
    s.sort()

    (cost, alignment) = lcs(a, s)

    if verbose:
        print
        print "input:  ", a
        print "sorted: ", s

        print 'cost:', cost
        print
        for (a,b) in alignment:
            print '  %16s => %s' % (a,b)
        print

    return [a for (a,b) in alignment if a is not None and b is not None]


if __name__ == '__main__':
    assert lcs('abc', '') == (3.0, [('a', None), ('b', None), ('c', None)])
    assert lcs('', 'abc') == (3.0, [(None, 'a'), (None, 'b'), (None, 'c')])
    assert lcs('abc', 'abc') == (0.0, [('a', 'a'), ('b', 'b'), ('c', 'c')])
    assert lcs('abcd', 'obce') == (4.0, [('a', None),
                                         (None, 'o'),
                                         ('b', 'b'),
                                         ('c', 'c'), 
                                         ('d', None),
                                         (None, 'e')])
    assert longest_increasing_subsequence([1, 10, 2, 2, 3, -1, 4, -5]) == [1, 2, 2, 3, 4]
    assert longest_increasing_subsequence([20, 1, 2, 2, 3, -1, 4]) == [1, 2, 2, 3, 4]
    
    print 'passed tests.'




US_state_abbrev = dict([
    ('AL', 'ALABAMA'),
    ('AK', 'ALASKA'),
    ('AS', 'AMERICAN SAMOA'),
    ('AZ', 'ARIZONA'),
    ('AR', 'ARKANSAS'),
    ('CA', 'CALIFORNIA'),
    ('CO', 'COLORADO'),
    ('CT', 'CONNECTICUT'),
    ('DE', 'DELAWARE'),
    ('DC', 'DISTRICT OF COLUMBIA'),
    ('FM', 'FEDERATED STATES OF MICRONESIA'),
    ('FL', 'FLORIDA'),
    ('GA', 'GEORGIA'),
    ('GU', 'GUAM'),
    ('HI', 'HAWAII'),
    ('ID', 'IDAHO'),
    ('IL', 'ILLINOIS'),
    ('IN', 'INDIANA'),
    ('IA', 'IOWA'),
    ('KS', 'KANSAS'),
    ('KY', 'KENTUCKY'),
    ('LA', 'LOUISIANA'),
    ('ME', 'MAINE'),
    ('MH', 'MARSHALL ISLANDS'),
    ('MD', 'MARYLAND'),
    ('MA', 'MASSACHUSETTS'),
    ('MI', 'MICHIGAN'),
    ('MN', 'MINNESOTA'),
    ('MS', 'MISSISSIPPI'),
    ('MO', 'MISSOURI'),
    ('MT', 'MONTANA'),
    ('NE', 'NEBRASKA'),
    ('NV', 'NEVADA'),
    ('NH', 'NEW HAMPSHIRE'),
    ('NJ', 'NEW JERSEY'),
    ('NM', 'NEW MEXICO'),
    ('NY', 'NEW YORK'),
    ('NC', 'NORTH CAROLINA'),
    ('ND', 'NORTH DAKOTA'),
    ('MP', 'NORTHERN MARIANA ISLANDS'),
    ('OH', 'OHIO'),
    ('OK', 'OKLAHOMA'),
    ('OR', 'OREGON'),
    ('PW', 'PALAU'),
    ('PA', 'PENNSYLVANIA'),
    ('PR', 'PUERTO RICO'),
    ('RI', 'RHODE ISLAND'),
    ('SC', 'SOUTH CAROLINA'),
    ('SD', 'SOUTH DAKOTA'),
    ('TN', 'TENNESSEE'),
    ('TX', 'TEXAS'),
    ('UT', 'UTAH'),
    ('VT', 'VERMONT'),
    ('VI', 'VIRGIN ISLANDS'),
    ('VA', 'VIRGINIA'),
    ('WA', 'WASHINGTON'),
    ('WV', 'WEST VIRGINIA'),
    ('WI', 'WISCONSIN'),
    ('WY', 'WYOMING')
])




## State or Territory
## USPS approved postal abbreviations
## GPO
## Traditional abbreviations
## Other abbreviations and codes

more_abbreviations = [
    ('Alabama', 'AL', 'Ala.', 'Ala.', None),
    ('Alaska', 'AK', 'Alaska', 'Alaska', 'Alas.'),
    ('Arizona', 'AZ', 'Ariz.', 'Ariz.', None),
    ('Arkansas', 'AR', 'Ark.', 'Ark.', 'A.R.'),
    ('California', 'CA', 'Calif.', 'Calif.', 'Cal., CF<sup>UCCG</sup>'),
    ('Colorado', 'CO', 'Colo.', 'Colo.', 'Col., CL<sup>UCCG</sup>'),
    ('Connecticut', 'CT', 'Conn.', 'Conn.', None),
    ('Delaware', 'DE', 'Del.', 'Del.', 'DL<sup>UCCG</sup>'),
    ('District of Columbia', 'DC', 'D.C.', 'D.C.', '(Federal district)'),
    ('Florida', 'FL', 'Fla.', 'Fla.', 'Flor.'),
    ('Georgia Georgia', 'GA', 'Ga.', 'Ga.', None),
    ('Hawaii', 'HI', 'Hawaii', 'Hawaii', 'H.I., HA<sup>UCCG</sup>'),
    ('Idaho', 'ID', 'Idaho', 'Idaho', 'Id. or Ida.'),
    ('Illinois', 'IL', 'Ill.', 'Ill.', 'Ills.'),
    ('Indiana', 'IN', 'Ind.', 'Ind.', None),
    ('Iowa', 'IA', 'Iowa', 'Iowa', 'Ia.'),
    ('Kansas', 'KS', 'Kans.', 'Kan.', 'KA<sup>UCCG</sup>'),
    ('Kentucky', 'KY', 'Ky.', 'Ky.', 'Ken. or Kent.'),
    ('Louisiana', 'LA', 'La.', 'La.', None),
    ('Maine', 'ME', 'Maine', 'Maine', 'Me.'),
    ('Maryland', 'MD', 'Md.', 'Md.', None),
    ('Massachusetts', 'MA', 'Mass.', 'Mass.', 'MS<sup>UCCG</sup>'),
    ('Michigan', 'MI', 'Mich.', 'Mich.', 'MC<sup>UCCG</sup>'),
    ('Minnesota', 'MN', 'Minn.', 'Minn.', None),
    ('Mississippi', 'MS', 'Miss.', 'Miss.', 'MI<sup>UCCG</sup>'),
    ('Missouri', 'MO', 'Mo.', 'Mo.', None),
    ('Montana', 'MT', 'Mont.', 'Mont.', None),
    ('Nebraska', 'NE', 'Nebr.', 'Neb.', 'NB<sup>UCCG</sup>'),
    ('Nevada', 'NV', 'Nev.', 'Nev.', None),
    ('New Hampshire', 'NH', 'N.H.', 'N.H.', None),
    ('New Jersey', 'NJ', 'N.J.', 'N.J.', None),
    ('New Mexico', 'NM', 'N. Mex.', 'N.M.', 'New M.'),
    ('New York', 'NY', 'N.Y.', 'N.Y.', 'N. York'),
    ('North Carolina', 'NC', 'N.C.', 'N.C.', 'N. Car.'),
    ('North Dakota', 'ND', 'N. Dak.', 'N.D.', None),
    ('Ohio', 'OH', 'Ohio', 'Ohio', 'O.'),
    ('Oklahoma', 'OK', 'Okla.', 'Okla.', None),
    ('Oregon', 'OR', 'Oreg.', 'Ore.', None),
    ('Pennsylvania', 'PA', 'Pa.', 'Pa.', 'Penn. or Penna.'),
    ('Rhode Island', 'RI', 'R.I.', 'R.I.', None),
    ('South Carolina', 'SC', 'S.C.', 'S.C.', 'S. Car.'),
    ('South Dakota', 'SD', 'S. Dak.', 'S.D.', None),
    ('Tennessee', 'TN', 'Tenn.', 'Tenn.', None),
    ('Texas', 'TX', 'Tex.', 'Texas', None),
    ('Utah', 'UT', 'Utah', 'Utah', 'Ut.'),
    ('Vermont', 'VT', 'Vt.', 'Vt.', None),
    ('Virginia', 'VA', 'Va.', 'Va.', None),
    ('Washington', 'WA', 'Wash.', 'Wash.', 'Wn., WN<sup>UCCG</sup>'),
    ('West Virginia', 'WV', 'W. Va.', 'W.Va.', 'W.V.'),
    ('Wisconsin', 'WI', 'Wis.', 'Wis.', 'Wisc., WS<sup>UCCG</sup>'),
    ('Wyoming', 'WY', 'Wyo.', 'Wyo.', None),
]



import atexit
import shelve
import cPickle as pickle
from functools import wraps


from datetime import datetime, timedelta
from copy import deepcopy
from threading import RLock


def timed_cache(seconds=0, minutes=0, hours=0, days=0):

    time_delta = timedelta( seconds=seconds,
                            minutes=minutes,
                            hours=hours,
                            days=days )

    def decorate(f):

        f._lock = RLock()
        f._updates = {}
        f._results = {}

        def do_cache(*args, **kwargs):

            lock = f._lock
            lock.acquire()

            try:
                key = (args, tuple(sorted(kwargs.items(), key=lambda i:i[0])))

                updates = f._updates
                results = f._results

                t = datetime.now()
                updated = updates.get(key, t)

                if key not in results or t-updated > time_delta:
                    # Calculate
                    updates[key] = t
                    result = f(*args, **kwargs)
                    results[key] = deepcopy(result)
                    return result

                else:
                    # Cache
                    return deepcopy(results[key])

            finally:
                lock.release()

        return do_cache

    return decorate

"""
if __name__ == "__main__":

    import time
    class T(object):
        @timed_cache(seconds=2)
        def expensive_func(self, c):
            time.sleep(.2)
            return c

    t = T ()
    for _ in xrange(30):
        time.sleep(.1)
        t1 = time.clock()
        print t.expensive_func('Calling expensive method')
        print "t - %i milliseconds"%int( (time.clock() - t1) * 1000. )
"""

class ShelfBasedCache(object):
    """ cache a function's return value to avoid recalulation and save cache in a shelve. """
    def __init__(self, func, key, None_is_bad=False):
        self.func = func
        self.filename = '{self.func.__name__}.shelf~'.format(self=self)
        self.cache = shelve.open(self.filename) #, writeback=True)
        self.key = key
        self.None_is_bad = None_is_bad
        self.__name__ = 'ShelfBasedCache(%s)' % func.__name__
    def __call__(self, *args):
        p_args = self.key(args)
        value = None
        recompute = True
        if self.cache.has_key(p_args):
            recompute = False
            value = self.cache[p_args]
            if value is None and self.None_is_bad:
                recompute = True
        if recompute:
            self.cache[p_args] = value = self.func(*args)
            self.cache.sync()
        return value

def persistent_cache(key=lambda x: x, None_is_bad=False):
    def wrap(f):
        return ShelfBasedCache(f, key, None_is_bad=None_is_bad)
    return wrap


# TODO:
#  * add option to pass a reference to another cache (maybe memcached client)
class memoize(object):
    """ cache a function's return value to avoid recalulation """
    def __init__(self, func):
        self.func = func
        self.cache = {}
        try:
            self.__name__ = func.__name__
            self.__doc__ = func.__doc__
        except AttributeError:
            pass
    def __call__(self, *args):
        try:
            return self.cache[args]
        except KeyError:
            value = self.func(*args)
            try:
                self.cache[args] = value
            except TypeError:
                # uncachable -- for instance, passing a list as an argument.
                raise TypeError('uncachable arguments %r passed to memoized function.' % (args,))
            return value
        except TypeError:
            # uncachable -- for instance, passing a list as an argument.
            raise TypeError('uncachable arguments %r passed to memoized function.' % (args,))
    def __repr__(self):
        return '<memoize(%r)>' % self.func


## TODO: automatically make a back-up of the pickle
class memoize_persistent(object):
    """
    cache a function's return value to avoid recalulation and save the
    cache (via pickle) at system exit so that it persists.

    WARNING: retrieves cache for functions which might not be equivalent
             if a revision is made to the code which is used to compute it.
    """
    def __init__(self, func, filename=None):
        self.func = func
        self.filename = filename or '{self.func.__name__}.cache.pkl~'.format(self=self)
        self.dirty = False
        self.key = 0
        self.cache = {}
        self.loaded = False
        atexit.register(self.save)

    def save(self):
        if self.cache and self.dirty:
            pickle.dump((self.cache, self.key), file(self.filename,'wb'))
            print '[ATEXIT] saved persistent cache for {self.func.__name__} to file "{self.filename}"'.format(self=self)
        else:
            print "[ATEXIT] found nothing to save in {self.func.__name__}'s cache.".format(self=self)

    def load(self):
        self.loaded = True
        loaded_key = None
        try:
            (cache, loaded_key) = pickle.load(file(self.filename,'r'))
        except IOError:
            pass
        finally:
            if self.key == loaded_key:
                self.cache = cache
                print 'loaded cache for {self.func.__name__}'.format(self=self)
            else:
                self.cache = {}
                print 'failed to load cache for {self.func.__name__}'.format(self=self)

    def __call__(self, *args):
        # wait until you call the function to un-pickle
        if not self.loaded:
            self.load()
        try:
            return self.cache[args]
        except KeyError:
            value = self.func(*args)
            try:
                self.cache[args] = value
            except TypeError:
                # uncachable -- for instance, passing a list as an argument.
                raise TypeError('uncachable arguments %r passed to memoized function.' % (args,))
            else:
                self.dirty = True
            return value
        except TypeError:
            # uncachable -- for instance, passing a list as an argument.
            raise TypeError('uncachable arguments %r passed to memoized function.' % (args,))

    def get_cached(self, *args):
        """ If result is cached return it, otherwise return `None`. """
        # wait until you call the function to un-pickle
        if not self.loaded:
            self.load()
        if args in self.cache:
            return self.cache[args]
        else:
            return None



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re
from arsenal.text import markup


_whitespace_cleanup = re.compile('[ ]*\n', re.MULTILINE)
def whitespace_cleanup(x):
    return _whitespace_cleanup.sub('\n', x)

def common_prefix(m):
    """
    Longest common prefix.

    >>> common_prefix(['prefix-a', \
                       'prefix-b', \
                       'prefix-c'])
    'prefix-'

    """
    if not m:
        return ''
    s1 = min(m)
    s2 = max(m)
    for i, c in enumerate(s1):
        if c != s2[i]:
            return s1[:i]
    return s1


# Borrowed from: http://www.codigomanso.com/en/2010/05/una-de-python-force_unicode/
def force_unicode(s, encoding='utf-8', errors='ignore'):
    """
    Returns a unicode object representing 's'. Treats bytestrings using the
    'encoding' codec.
    """
    if s is None:
        return u''
    try:
        if not isinstance(s, basestring,):
            if hasattr(s, '__unicode__'):
                s = unicode(s)
            else:
                try:
                    s = unicode(str(s), encoding, errors)
                except UnicodeEncodeError:
                    if not isinstance(s, Exception):
                        raise
                    # If we get to here, the caller has passed in an Exception
                    # subclass populated with non-ASCII data without special
                    # handling to display as a string. We need to handle this
                    # without raising a further exception. We do an
                    # approximation to what the Exception's standard str()
                    # output should be.
                    s = ' '.join(force_unicode(arg, encoding, errors) for arg in s)
        elif not isinstance(s, unicode):
            # Note: We use .decode() here, instead of unicode(s, encoding,
            # errors), so that if s is a SafeString, it ends up being a
            # SafeUnicode at the end.
            s = s.decode(encoding, errors)
    except UnicodeDecodeError as e:
        if not isinstance(s, Exception):
            raise UnicodeDecodeError (s, *e.args)
        else:
            # If we get to here, the caller has passed in an Exception
            # subclass populated with non-ASCII bytestring data without a
            # working unicode method. Try to handle this without raising a
            # further exception by individually forcing the exception args
            # to unicode.
            s = ' '.join(force_unicode(arg, encoding, errors) for arg in s)
    return unicode(s)


def PRE(x):
    return '<pre>' +  x.replace('&', '&amp;').replace('<', '&lt;').replace('>', '&gt;') + '</pre>'


# native, HTML, default Unicode (Code page 850), Unicode combined Character, Windows-1250
"""
_recodings = {'ae': ['ä', u'ä', '&auml;', '\u00E4', u'\u00E4', '\u0308a', '\xc3\xa4'],
              'oe': ['ö', u'ö', '&ouml;', '\u00F6', u'\u00F6', '\u0308o', '\xc3\xb6'],
              'ue': ['ü', u'ü', '&uuml;', '\u00FC', u'\u00FC', '\u0308u', '\xc3\xbc'],
              'Ae': ['Ä', u'Ä', '&Auml;', '\u00C4', u'\u00C4', '\u0308A', '\xc3\x84'],
              'Oe': ['Ö', u'Ö', '&Ouml;', '\u00D6', u'\u00D6', '\u0308O', '\xc3\x96'],
              'Ue': ['Ü', u'Ü', '&Uuml;', '\u00DC', u'\u00DC', '\u0308U', '\xc3\x9c'],
              'ss': ['ß', u'ß', '&szlig;', '\u00DF', u'\u00DF', '\xc3\x9f'],
              'e': ['é', u'é', '\xc3\xa9'],
             }
"""

# taken from NLTK
def htmltotext(x):
    """ Remove HTML markup from the given string. """
    # remove inline JavaScript / CSS
    x = re.compile('<script.*?>[\w\W]*?</script>', re.IGNORECASE).sub('', x)
    x = re.compile('<style.*?>[\w\W]*?</style>', re.IGNORECASE).sub('', x)
    # remove html comments. must be done before removing regular tags since comments can contain '>' characters.
    x = re.sub(r'<!--([\w\W]*?)-->', '', x)
    # remove the remaining tags
    x = re.sub(r'(?s)<.*?>', ' ', x)
    # remove html entities
    x = markup.remove_entities(x)
    # clean up whitespace
    x = re.sub('[ ]+', ' ', x)
    x = re.compile('(\n\n+)', re.MULTILINE).sub('\n', x)
    return x


import unicodedata
def strip_accents(s):
    """ Transform accentuated unicode symbols into their simple counterpart. """
    return u''.join(c for c in unicodedata.normalize('NFKD', s) if not unicodedata.combining(c))

# iso-8859-1
LATIN2ASCII = {
    # uppercase
    u'\u00c0': 'A`',
    u'\u00c1': "A'",
    u'\u00c2': 'A^',
    u'\u00c3': 'A~',
    u'\u00c4': 'A:',
    u'\u00c5': 'A%',
    u'\u00c6': 'AE',
    u'\u00c7': 'C,',
    u'\u00c8': 'E`',
    u'\u00c9': "E'",
    u'\u00ca': 'E^',
    u'\u00cb': 'E:',
    u'\u00cc': 'I`',
    u'\u00cd': "I'",
    u'\u00ce': 'I^',
    u'\u00cf': 'I:',
    u'\u00d0': "D'",
    u'\u00d1': 'N~',
    u'\u00d2': 'O`',
    u'\u00d3': "O'",
    u'\u00d4': 'O^',
    u'\u00d5': 'O~',
    u'\u00d6': 'O:',
    u'\u00d8': 'O/',
    u'\u00d9': 'U`',
    u'\u00da': "U'",
    u'\u00db': 'U~',
    u'\u00dc': 'U:',
    u'\u00dd': "Y'",
    u'\u00df': 'ss',
    # lowercase
    u'\u00e0': 'a`',
    u'\u00e1': "a'",
    u'\u00e2': 'a^',
    u'\u00e3': 'a~',
    u'\u00e4': 'a:',
    u'\u00e5': 'a%',
    u'\u00e6': 'ae',
    u'\u00e7': 'c,',
    u'\u00e8': 'e`',
    u'\u00e9': "e'",
    u'\u00ea': 'e^',
    u'\u00eb': 'e:',
    u'\u00ec': 'i`',
    u'\u00ed': "i'",
    u'\u00ee': 'i^',
    u'\u00ef': 'i:',
    u'\u00f0': "d'",
    u'\u00f1': 'n~',
    u'\u00f2': 'o`',
    u'\u00f3': "o'",
    u'\u00f4': 'o^',
    u'\u00f5': 'o~',
    u'\u00f6': 'o:',
    u'\u00f8': 'o/',
    u'\u00f9': 'o`',
    u'\u00fa': "u'",
    u'\u00fb': 'u~',
    u'\u00fc': 'u:',
    u'\u00fd': "y'",
    u'\u00ff': 'y:',
}


LIGATURES_PAIRS = [
    (u'ﬃ', 'ffi'),
    (u'ﬀ', 'ff'),
    (u'ﬁ', 'fi'),
    (u'ﬂ', 'fl'),

    (u'—', '--'),
    (u'–', '-'),

    (u'“', '"'),
    (u'”', '"'),

    (u" ", " "),
    (u"’", "'"),
    (u"•", "*"),
    (u"…", "..."),

    (u'\u00c6', 'AE'),
    (u'\u00e6', 'ae'),
    (u'\u0152', 'OE'),
    (u'\u0153', 'oe'),
    (u'\u0132', 'IJ'),
    (u'\u0133', 'ij'),
    (u'\u1d6b', 'ue'),
    (u'\ufb00', 'ff'),
    (u'\ufb01', 'fi'),
    (u'\ufb02', 'fl'),
    (u'\ufb03', 'ffi'),
    (u'\ufb04', 'ffl'),
    (u'\ufb05', 'ft'),
    (u'\ufb06', 'st'),
]

LIGATURES_MAP = dict(LIGATURES_PAIRS)

LIGATURES = re.compile(u'(%s)' % u'|'.join(k for k,v in LIGATURES_PAIRS))


#import unicodedata
#>>> source = u'Mikael Håfström'
#>>> unicodedata.normalize('NFKD', source).encode('ascii', 'ignore')
def normalize(text):
    u"""
    >>> normalize(u'Efﬁciently')
    u'Efficiently'
    """
    text = force_unicode(text)
    return unicodedata.normalize('NFKD', text) #.encode('ascii', 'ignore')


def remove_ligatures(text):
    u"""
    >>> remove_ligatures(u'Efﬁciently')
    u'Efficiently'
    """
    text = normalize(text)   # or at least force_unicode
    return LIGATURES.sub(lambda m: LIGATURES_MAP[m.group(1)], text)


def convert_special_html_escapes(text):
    for plain, funny in (('ä','&auml;'), ('ö','&ouml;'), ('ü','&uuml;'), ('Ä','&Auml;'), ('Ö','&Ouml;'),
                         ('Ü','&Uuml;'), ('ß','&szlig;')):
        text = text.replace(funny, plain)
    return text


def remove_html_escapes(text):
    for plain, funny in (('&','&amp;'), ('<','&lt;'), ('>','&gt;'), ('"','&quot;'), ("'",'&#39;')):
        text = text.replace(funny, plain)
    return text


def remove_latin(text):
    text = text.encode('utf-8')
    text = unicode(text)
    # chars that ISO-8859-1 does not support
    for plain, funny_set in (
                             ('"', u'“”'),
                             ('-', u'\u2013\u2014\u2022'),
                             ("'", u'\u2018\u2019'),
                             ('', u'\ufffd\u2122\u2020'),
                             ('...', u'\u2026'),
#                             ('i', u'\u012b'),
#                             ('ã', u'\u0101'),
#                             ('r', u'\u0159'),
#                             ('Z',u'\u017d'),
#                             ('z', u'\u017e'),
                             ('EUR', u'\u20ac')):
        for funny in funny_set:
            orig = text
            text = text.replace(funny, plain)
            if orig != text:
                print 'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX'
                print funny, '->', plain
                print list(funny_set)
                print text
                print 'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX'
    return text

def remove_accents(text):
    text = unicode(text).encode('utf-8')
    for plain, funny_set in (('a','áàâãäå\u0101'), ('e','éèêẽë'), ('i',"íìîĩï"), ('o','óòôõöø'),
                             ('u',"úùûũü"), ('A','ÁÀÂÃÄÅ'), ('E','ÉÈÊẼË'), ('I',"ÍÌÎĨÏ"),
                             ('O','ÓÒÔÕÖØ'), ('U',"ÚÙÛŨÜ"), ('n',"ñ"), ('c',"ç"), ('N',"Ñ"),
                             ('C',"Ç"), ('d',"Þ"), ('ss',"ß"), ('ae',"æ"), ('oe','œ')):
        for funny in funny_set:
            text = text.replace(funny, plain)
    return text



if __name__ == '__main__':
    import sys

    if sys.stdin.isatty():
        sys.exit(1)

    x = sys.stdin.read()
    #x = htmltotext(x)

    x = remove_ligatures(x).encode('ascii', 'ignore')

    print x



def complain(self, *wargs, **kwargs):
    if not hasattr(self, 'warning'):
        raise AssertionError
    if isinstance(self.warning, Exception):
        raise self.warning
    elif isinstance(self.warning, basestring):
        raise AssertionError(self.warning)
    else:
        raise AttributeError, self.warning


class ImmutableList(tuple):
    """
    A tuple-like object that raises useful errors when it is asked to mutate.

    Example::

        >>> a = ImmutableList(range(5))
        >>> a[3] = '4'
        Traceback (most recent call last):
            ...
        AssertionError
    """

    def __new__(cls, *args, **kwargs):
        return tuple.__new__(cls, *args, **kwargs)

    # All list mutation functions complain.
    __delitem__  = complain
    __delslice__ = complain
    __iadd__     = complain
    __imul__     = complain
    __setitem__  = complain
    __setslice__ = complain
    append       = complain
    extend       = complain
    insert       = complain
    pop          = complain
    remove       = complain
    sort         = complain
    reverse      = complain
    __setattr__  = complain
    __delattr__  = complain
    __mul__      = complain
    __rmul__     = complain


class InequalityComplaints(object):
    """ mixin which complains about greater and lessthan comparisons. """
    __ge__ = complain
    __gt__ = complain
    __le__ = complain
    __lt__ = complain


class SaferList(list, InequalityComplaints):
    """
    Immutable list which also ensures that slices have indicies strictly in bounds.
    """

    def __getitem__(self, item):
        assert self.boundscheck(item)
        return list.__getitem__(self, item)

    def boundscheck(self, item):
        return isinstance(item, int) and item >= 0 and item < list.__len__(self)

    def __getslice__(self, i, j):
        assert map(self.boundscheck, filter(None, (i,j))), 'getslice does not have indicies in bounds. %r' % ((i,j),)
        xx = list.__getslice__(self, i, j)
        assert len(xx) == (j-i), 'slices must have the proper size!'
        return SaferList(xx)

    append       = complain
    pop          = complain
    __setattr__  = complain
    __setitem__  = complain
    __setslice__ = complain
    __add__      = complain
    __contains__ = complain
    __delattr__  = complain
    __delitem__  = complain
    __delslice__ = complain
    __ge__       = complain
    __gt__       = complain
    __hash__     = complain
    __iadd__     = complain
    __imul__     = complain
    __le__       = complain
    __lt__       = complain
    __mul__      = complain
    __rmul__     = complain
    extend       = complain
    insert       = complain
    pop          = complain
    remove       = complain
    reverse      = complain
    sort         = complain

    # TODO: these methods might need special attention
    __eq__       = complain
    __ne__       = complain


# TODO: implement ImmutableDict
"""
class ImmutableDict(object):    
    @classmethod
    def fromkeys(cls, keys, value=None):
        instance = super(cls, cls).__new__(cls)
        instance.__init__(zip(keys, repeat(value)))
        return instance

    def __reduce_ex__(self, protocol):
        return type(self), (dict(self),)

    setdefault  = complain
    update      = complain
    pop         = complain
    popitem     = complain
    __setitem__ = complain
    __delitem__ = complain
    clear       = complain
""" 


if __name__ == '__main__':

    # TODO: add some tests
    def example():
        
        x = ImmutableList([])

    example()



from itertools import imap, ifilter

from arsenal.nlp.lexicon.englishwords import englishwords as dictionary

def overlap(collection):
    for w in ifilter(dictionary.__contains__, collection):
        print w

def parse(lexicon):
    return map(str.lower, imap(str.strip, lexicon.split()))


if __name__ == '__main__':

    import male
    #overlap(parse(male.male_names))

    import female
    #overlap(parse(female.female_names))

    import last
    #overlap(parse(last.last_names))

    from first_name_stats import names




# TODO:
#  - consider allowing lazy things to depend on one another

from types import GeneratorType

class lazy(object):
    """
    Lazily load a property defined by a method. The method wrapped is called
    at most once to retrieve the result and the result is reused. If the method
    is a generator, the value is stored as a list.

    Note: instances must have a `__dict__` attribute in order for this property
    to work, i.e. no '__slots__' class attribute.

    Implementation detail: this is not implemented as a data descriptor so that
    we can completely avoid the function call overhead. If one choses to invoke
    `__get__` by hand the property will still work as expected because the 
    lookup logic is replicated in `__get__` for manual invocation.
    """

    def __init__(self, func):
        self.__name__ = func.__name__
        self.__module__ = func.__module__
        self.__doc__ = func.__doc__
        self.func = func

    def __get__(self, obj, type_=None):
        if obj is None:
            return self
        try:
            value = obj.__dict__[self.__name__]
        except KeyError:
            value = self.func(obj)
            if isinstance(value, GeneratorType):   # store generators as lists
                value = list(value)
            obj.__dict__[self.__name__] = value
        return value

    def __set__(self, obj, value):
        raise NotImplementedError

    def __delete__(self, obj):
        raise NotImplementedError



import sys, os, commands

def whatismyip2():
    """
    Alternative implementation of whatismyip based on different hacks.

    Implementation of this function borrowed from:
        woof -- an ad-hoc single file webserver
        Copyright (C) 2004-2009 Simon Budig  <simon@budig.de>
    """
    import socket
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
        s.connect(("gmail.com",80))
        ip = s.getsockname()[0]
    except: #socket.SocketError:
        return None
    finally:
        s.close()
    return ip

def whatismyip():
    """
    Utility function to guess the IP (as a string) where the server can be
    reached from the outside. Quite nasty problem actually.

    Implementation of this function borrowed from:
        woof -- an ad-hoc single file webserver
        Copyright (C) 2004-2009 Simon Budig  <simon@budig.de>

    """
    if sys.platform == "cygwin":
        ipcfg = os.popen("ipconfig").readlines()
        for l in ipcfg:
            try:
                candidat = l.split(":")[1].strip()
                if candidat[0].isdigit():
                    break
            except:
                pass
        return candidat

    os.environ["PATH"] = "/sbin:/usr/sbin:/usr/local/sbin:" + os.environ["PATH"]
    platform = os.uname()[0]

    if platform == "Linux":
        netstat = commands.getoutput("LC_MESSAGES=C netstat -rn")
        defiface = [i.split()[-1] for i in netstat.split('\n') if i.split()[0] == "0.0.0.0"]
    elif platform in ("Darwin", "FreeBSD", "NetBSD"):
        netstat = commands.getoutput("LC_MESSAGES=C netstat -rn")
        defiface = [i.split()[-1] for i in netstat.split('\n') if len(i) > 2 and i.split()[0] == "default"]
    elif platform == "SunOS":
        netstat = commands.getoutput("LC_MESSAGES=C netstat -arn")
        defiface = [i.split()[-1] for i in netstat.split('\n') if len(i) > 2 and i.split()[0] == "0.0.0.0"]
    else:
        print >>sys.stderr, "Unsupported platform: please add support for your platform in find_ip()."
        return None

    if not defiface:
        return None

    if platform == "Linux":
        ifcfg = commands.getoutput("LC_MESSAGES=C ifconfig " + defiface[0]).split("inet addr:")
    elif platform in ("Darwin", "FreeBSD", "SunOS", "NetBSD"):
        ifcfg = commands.getoutput("LC_MESSAGES=C ifconfig " + defiface[0]).split("inet ")

    if len(ifcfg) != 2:
        return None
    ip_addr = ifcfg[1].split()[0]

    # sanity check
    try:
        ints = [i for i in ip_addr.split(".") if 0 <= int(i) <= 255]
        if len(ints) != 4:
            return None
    except ValueError:
        return None

    return ip_addr


if __name__ == '__main__':
    print 'whatismyip: ', whatismyip()
    print 'whatismyip2:', whatismyip2()



data = """
#*****************************************************************************
# ConvertAll, a units conversion program
# Copyright (C) 2005, Douglas W. Bell
#
# This is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License, Version 2.  This program is
# distributed in the hope that it will be useful, but WITTHOUT ANY WARRANTY.
#*****************************************************************************
#
# Units are defined by an optional quantity and an equivalent unit or unit
# combination.  A python expression may be used for the quantity, but is
# resticted to using only the following operators: *, /, +, -, **, (, ).
# Beware of integer division truncation: be sure to use a float for at
# least one of the values.
#
# The unit type must be placed in square brackets before a set of units.
# The first comment after the equivalent unit will be put in parenthesis after
# the unit name (usually used to give the full name of an abbreviated unit).
# The next comment will be used in the program list's comment column;
# later comments and full line comments are ignored.
#
# Non-linear units are indicated with an equivalent unit in square brackets,
# followed by either equations or equivalency lists for the definition.
# For equations, two are given, separated by a ';'.  Both are functions of
# "x", the first going from the unit to the equivalent unit and the second
# one in reverse.  Any valid Python expression returning a float (including 
# the functions in the math module) should work.  The equivalency list is a 
# python list of tuples giving points for linear interpolation.
#
# All units must reduce to primitive units, which are indicated by an '!'
# as the equivalent unit.  Circular refernces must also be avoided.
#
# Primitive units:  kg, m, s, K, A, mol, cd, rad, sr, bit, unit
#
##############################################################################

#
# mass units
#
[mass]
kg                = !                  # kilogram
kilogram          = kg
key               = kg                 # # drug slang
hectogram         = 100 gram
dekagram          = 10 gram
gram              = 0.001 kg
g                 = gram               # gram
decigram          = 0.1 gram
centigram         = 0.01 gram
milligram         = 0.001 gram
mg                = milligram          # milligram
microgram         = 0.001 mg
tonne             = 1000 kg            # # metric
metric ton        = tonne
megagram          = tonne
kilotonne         = 1000 tonne         # # metric
gigagram          = 1e9 gram
teragram          = 1e12 gram
carat             = 0.2 gram
ct                = carat              # carat
amu               = 1.66053873e-27 kg  # atomic mass
atomic mass unit  = amu
pound             = 0.45359237 kg
lb                = pound              # pound
lbm               = pound              # pound
ounce             = 1/16.0 pound
oz                = ounce              # ounce
lid               = ounce              # # drug slang
ton               = 2000 lb            # # non-metric
kiloton           = 1000 ton           # # non-metric
slug              = lbf*s^2/ft
stone             = 14 lb
grain             = 1/7000.0 lb


#
# length / distance units
#
[length]
m                    = !              # meter
meter                = m
metre                = m
decimeter            = 0.1 m
cm                   = 0.01 m         # centimeter
centimeter           = cm
mm                   = 0.001 m        # millimeter
millimeter           = mm
micrometer           = 1e-6 m
micron               = micrometer
nanometer            = 1e-9 m
nm                   = nanometer      # nanometer
dekameter            = 10 m
hectometer           = 100 m
km                   = 1000 m         # kilometer
kilometer            = km
megameter            = 1000 km
angstrom             = 1e-10 m
fermi                = 1e-15 m        # # nuclear sizes
inch                 = 2.54 cm
in                   = inch           # inch
inches               = inch
mil                  = 0.001 inch
microinch            = 1e-6 inch
microinches          = microinch
foot                 = 12 inch
ft                   = foot           # foot
feet                 = foot
yard                 = 3 ft
yd                   = yard           # yard
mile                 = 5280 ft
mi                   = mile           # mile
nautical mile        = 1852 m
nmi                  = nautical mile  # nautical mile
league               = 3 mile
chain                = 66 ft
fathom               = 6 ft
rod                  = 5.5 yard
furlong              = 40 rod
hand                 = 4 inch
cubit                = 21.8 inch      # # biblical unit
point                = 1/72.27 inch
pica                 = 12 point
caliber              = 0.01 inch      # # bullet sizes
football field       = 100 yd
marathon             = 46145 yd
au                   = 1.49597870691e11 m   # astronomical unit
astronomical unit    = au
light year           = 365.25 light speed * day
light minute         = light speed * min
light second         = light speed * s
parsec               = 3.0856775813e16 m
kiloparsec           = 1000 parsec
megaparsec           = 1000 kiloparsec
screw size           = [in] 0.013*x + 0.06 ; (x - 0.06) / 0.013 \
                       # # Unified diameters, non-linear
AWG                  = [in] 92.0**((36-x)/39.0)/200.0 ; \
                       36 - 39.0*log(200.0*x)/log(92.0) \
                       # American Wire Gauge \
                       # use -1, -2 for 00, 000; non-linear
American Wire Gauge  = [in] 92.0**((36-x)/39.0)/200.0 ; \
                       36 - 39.0*log(200.0*x)/log(92.0) \
                       #  # use -1, -2 for 00, 000; non-linear
standard gauge       = [in] [(-5, .448350), (1, .269010), (14, .0747250), \
                       (16, .0597800), (17, .0538020), (20, .0358680), \
                       (26, .0179340), (31, .0104615), (36, .00672525), \
                       (38, .00597800)] # steel \
                       # Manufacturers Std. Gauge, non-linear
zinc gauge           = [in] [(1, .002), (10, .02), (15, .04), (19, .06), \
                       (23, .1), (24, .125), (27, .5), (28, 1)]  \
                       # # sheet metal thickness, non-linear
ring size            = [in] 0.1018*x + 1.4216 ; (x - 1.4216) / 0.1018  \
                       # # US size, circum., non-linear
shoe size mens       = [in] x/3.0 + 7 + 1/3.0 ; (x - 7 - 1/3.0) * 3 \
                       # # US sizes, non-linear
shoe size womens     = [in] x/3.0 + 6 + 5/6.0 ; (x - 6 - 5/6.0) * 3 \
                       # # US sizes, non-linear


#
# time units
#
[time]
s             = !                 # second
sec           = s                 # second
second        = s
ms            = 0.001 s           # millisecond
millisecond   = ms
microsecond   = 1e-6 s
ns            = 1e-9 s            # nanosecond
nanosecond    = ns
minute        = 60 s
min           = minute            # minute
hour          = 60 min
hr            = hour              # hour
bell          = 30 min            #  # naval definition
watch         = 4 hour
watches       = watch
day           = 24 hr
week          = 7 day
wk            = week              # week
fortnight     = 14 day
month         = 1/12.0 year
year          = 365.242198781 day
yr            = year              # year
calendar year = 365 day
decade        = 10 year
century       = 100 year
centuries     = century
millennium    = 1000 year
millennia     = millennium
[scheduling]
man hour      = 168/40.0 hour
man week      = 40 man hour
man month     = 1/12.0 man year
man year      = 52 man week


#
# temperature
#
[temperature]
K                 = !     # Kelvin
Kelvin            = K
deg K             = K     # Kelvin
degree Kelvin     = K

C                 = [K] x + 273.15 ; x - 273.15  # Celsius  # non-linear
Celsius           = [K] x + 273.15 ; x - 273.15  #          # non-linear
deg C             = [K] x + 273.15 ; x - 273.15  # Celsius  # non-linear
degree Celsius    = [K] x + 273.15 ; x - 273.15  #          # non-linear

R                 = 5/9.0 K     # Rankine
Rankine           = R
deg R             = R           # Rankine
F                 = [R] x + 459.67 ; x - 459.67  # Fahrenheit  # non-linear
Fahrenheit        = [R] x + 459.67 ; x - 459.67  #             # non-linear
deg F             = [R] x + 459.67 ; x - 459.67  # Fahrenheit  # non-linear
degree Fahrenheit = [R] x + 459.67 ; x - 459.67  #             # non-linear

[temp. diff.]
C deg             = K        # Celsius degree
Celsius degree    = C deg
F deg             = R        # Fahrenheit deg.
Fahrenheit degree = F deg


#
# electrical units
#
[current]
A              = !              # ampere
ampere         = A
amp            = A
milliampere    = 0.001 A
milliamp       = milliampere
mA             = milliampere    # milliampere
microampere    = 0.001 mA
kiloampere     = 1000 A
kA             = kiloampere     # kiloampere
[charge]
coulomb        = A*s
amp hour       = A*hr
mAh            = 0.001 amp hour # milliamp hour
milliamp hour  = mAh
[potential]
volt           = W/A
V              = volt           # volt
millivolt      = 0.001 volt
mV             = millivolt      # millivolt
kilovolt       = 1000 volt
kV             = kilovolt       # kilovolt
[resistance]
ohm            = V/A
milliohm       = 0.001 ohm
microhm        = 0.001 milliohm
kilohm         = 1000 ohm
[conductance]
siemens        = A/V
[capacitance]
farad          = coulomb/V
millifarad     = 0.001 farad
microfarad     = 0.001 millifarad
nanofarad      = 1e-9 farad
picofarad      = 1e-12 farad
[magn. flux]
weber          = V*s
Wb             = weber          # weber
[inductance]
henry          = Wb/A
H              = henry          # henry
millihenry     = 0.001 henry
mH             = millihenry     # millihenry
microhenry     = 0.001 mH
[flux density]
tesla          = Wb/m^2
T              = tesla          # tesla


#
# molecular units
#
[molecular qty]
mol          = !           # mole       # gram mole
mole         = mol         #            # gram mole
gram mole    = mol
kilomole     = 1000 mol
kmol         = kilomole    # kilomole
pound mole   = mol*lbm/gram
lbmol        = pound mole  # pound mole
[size of a mol]
avogadro     = gram/amu*mol


#
# Illumination units
#
[lum. intens.]
cd          = !          # candela
candela     = cd

[luminous flux]
lumen        = cd * sr
lm           = lumen     # lumen

[illuminance]
lux          = lumen/m^2
footcandle   = lumen/ft^2
metercandle  = lumen/m^2

[luminance]
lambert      = cd/pi*cm^2
millilambert = 0.001 lambert
footlambert  = cd/pi*ft^2


#
# angular units
#
[angle]
radian      = !
rad         = radian         # radian
circle      = 2 pi*radian
turn        = circle
revolution  = circle
rev         = revolution     # revolution
degree      = 1/360.0 circle
deg         = degree         # degree
arc min     = 1/60.0 degree  # minute
arc minute  = arc min
min arc     = arc min        # minute
minute arc  = arc min
arc sec     = 1/60.0 arc min # second
arc second  = arc sec
sec arc     = arc sec        # second
second arc  = arc sec
quadrant    = 1/4.0 circle
right angle = quadrant
gradian     = 0.01 quadrant


#
# solid angle units
#
[solid angle]
sr         = !      # steradian
steradian  = sr
sphere     = 4 pi*sr
hemisphere = 1/2.0 sphere


#
# information units
#
[data]
bit       = !
kilobit   = 1000 bit   #           # based on power of 10
byte      = 8 bit
B         = byte       # byte
kilobyte  = 1024 byte  #           # based on power of 2
kB        = kilobyte   # kilobyte  # based on power of 2
megabyte  = 1024 kB    #           # based on power of 2
MB        = megabyte   # megabyte  # based on power of 2
gigabyte  = 1024 MB    #           # based on power of 2
GB        = gigabyte   # gigabyte  # based on power of 2
terabyte  = 1024 GB    #           # based on power of 2
TB        = terabyte   # terabyte  # based on power of 2
petabyte  = 1024 TB    #           # based on power of 2
PB        = petabyte   # petabyte  # based on power of 2

kibibyte  = 1024 byte
KiB       = kibibyte   # kibibyte
mebibyte  = 1024 KiB
MiB       = mebibyte   # mebibyte
gibibyte  = 1024 MiB
GiB       = gibibyte   # gibibyte
tebibyte  = 1024 GiB
TiB       = tebibyte   # tebibyte
pebibyte  = 1024 TiB
PiB       = pebibyte   # pebibyte

[data transfer]
bps       = bit/sec    # bits / second
kbps      = 1000 bps   # kilobits / sec.  # based on power of 10


#
# Unitless numbers
#
[quantity]
unit               = !
1                  = unit            # unit
pi                 = 3.14159265358979323846 unit
pair               = 2 unit
hat trick          = 3 unit          # # sports
dozen              = 12 unit
doz                = dozen           # dozen
bakers dozen       = 13 unit
score              = 20 unit
gross              = 144 unit
great gross        = 12 gross
ream               = 500 unit
percent            = 0.01 unit
%                  = percent
mill               = 0.001 unit
[interest rate]
APR                = [unit] log(1 + x/100) ;  (exp(x) - 1)*100 \
                     # annual % rate # based on continuous compounding
[concentration]
proof              = 1/200.0 unit    # # alcohol content
ppm                = 1e-6 unit       # parts per million
parts per million  = ppm
ppb                = 1e-9 unit       # parts per billion
parts per billion  = ppb
ppt                = 1e-12 unit      # parts per trillion
parts per trillion = ppt
karat              = 1/24.0 unit     # # gold purity
carat gold         = karat           # # gold purity


#
# force units
#
[force]
newton         = kg*m/s^2
N              = newton          # newton
dekanewton     = 10 newton
kilonewton     = 1000 N
kN             = kilonewton      # kilonewton
meganewton     = 1000 kN
millinewton    = 0.001 N
dyne           = cm*g/s^2
kg force       = kg * gravity    # kilogram f
kgf            = kg force        # kilogram force
kilogram force = kg force
gram force     = g * gravity
pound force    = lbm * gravity
lbf            = pound force     # pound force
ton force      = ton * gravity
ounce force    = ounce * gravity
ozf            = ounce force     # ounce force


#
# area units
#
[area]
barn          = 1e-28 m^2       # # particle physics
are           = 100 m^2
decare        = 10 are
dekare        = 10 are
hectare       = 100 are
acre          = 10 chain^2
section       = mile^2
township      = 36 section
homestead     = 160 acre
circular inch = 1/4.0 pi*in^2   # # area of 1 inch circle
circular mil  = 1/4.0 pi*mil^2  # # area of 1 mil circle


#
# volume units
#
[volume]
cc                   = cm^3                 # cubic centimeter
cubic centimeter     = cc
liter                = 1000 cc
l                    = liter                # liter
litre                = liter
deciliter            = 0.1 liter
centiliter           = 0.01 liter
milliliter           = cc
ml                   = milliliter           # milliliter
dekaliter            = 10 liter
hectoliter           = 100 liter
kiloliter            = 1000 liter
kl                   = kiloliter            # kiloliter
megaliter            = 1000 kiloliter
gallon               = 231 in^3             #             # US liquid
gal                  = gallon               # gallon      # US liquid
quart                = 1/4.0 gallon         #             # US liquid
qt                   = quart                # quart       # US liquid
pint                 = 1/2.0 quart          #             # US liquid
pt                   = pint                 # pint        # US liquid
fluid ounce          = 1/16.0 pint          #             # US
fl oz                = fluid ounce          # fluid ounce # US
ounce fluid          = fluid ounce          #             # US
imperial gallon      = 4.54609 liter
imp gal              = imperial gallon      # imperial gallon
imperial quart       = 1/4.0 imp gal
imp qt               = imperial quart       # imperial quart
imperial pint        = 1/8.0 imp gal
imp pt               = imperial pint        # imperial pint
imperial fluid ounce = 1/160.0 imp gal
imp fl oz            = imperial fluid ounce # imperial fluid ounce
cup                  = 8 fl oz
tablespoon           = 1/16.0 cup
tbsp                 = tablespoon           # tablespoon
teaspoon             = 1/3.0 tbsp
tsp                  = teaspoon             # teaspoon
barrel               = 42 gallon
bbl                  = barrel               # barrel
shot                 = 1.5 fl oz
fifth                = 1/5.0 gallon         #             # alcohol
wine bottle          = 750 ml
magnum               = 1.5 liter            #             # alcohol
keg                  = 15.5 gallon          #             # beer
bushel               = 2150.42 in^3
peck                 = 1/4.0 bushel
cord                 = 128 ft^3
board foot           = ft^2*in
board feet           = board foot


#
# velocity units
#
[velocity]
knot        = nmi/hr
kt          = knot             # knot
light speed = 2.99792458e8 m/s
mph         = mi/hr            # miles/hour
kph         = km/hr            # kilometers/hour
mach        = 331.46 m/s       # # speed sound at STP
[rot. velocity]
rpm         = rev/min          # rev/min
rps         = rev/sec          # rev/sec


#
# flow rate units
#
[fluid flow]
gph         = gal/hr           # gallons/hour
gpm         = gal/min          # gallons/minute
cfs         = ft^3/sec         # cu ft/second
cfm         = ft^3/min         # cu ft/minute
lpm         = l/min            # liter/min
[gas flow]
sccm        = atm*cc/min       # std cc/min      # pressure * flow
sccs        = atm*cc/sec       # std cc/sec      # pressure * flow
slpm        = atm*l/min        # std liter/min   # pressure * flow
slph        = atm*l/hr         # std liter/hour  # pressure * flow
scfh        = atm*ft^3/hour    # std cu ft/hour  # pressure * flow
scfm        = atm*ft^3/min     # std cu ft/min   # pressure * flow


#
# pressure units
#
[pressure]
Pa                    = N/m^2                    # pascal
pascal                = Pa
kPa                   = 1000 Pa                  # kilopascal
kilopascal            = kPa
MPa                   = 1000 kPa                 # megapascal
megapascal            = MPa
atm                   = 101325 Pa                # atmosphere
atmosphere            = atm
bar                   = 1e5 Pa
mbar                  = 0.001 bar                # millibar
millibar              = mbar
microbar              = 0.001 mbar
decibar               = 0.1 bar
kilobar               = 1000 bar
mm Hg                 = mm*density Hg*gravity
millimeter of Hg      = mm Hg
torr                  = mm Hg
in Hg                 = in*density Hg*gravity    # inch of Hg
inch of Hg            = in Hg
m water               = m*density water*gravity  # meter of H2O
m H2O                 = m water                  # meter of H2O
meter of water        = m water
in water              = in*density water*gravity # inch of H2O
in H2O                = in water                 # inch of H2O
inch of water         = in water
ft water              = ft*density water*gravity # feet of H2O
ft H2O                = ft water                 # feet of H20
feet of water         = ft water
foot of head          = ft water
ft hd                 = ft water                 # foot of head
psi                   = lbf/in^2                 # pound / sq inch
pound per sq inch     = psi
ksi                   = 1000 psi                 # 1000 lb / sq inch


#
# density units
#
[density]
density water         = gram/cm^3
density sea water     = 1.025 gram/cm^3
density Hg            = 13.5950981 gram/cm^3
density air           = 1.293 kg/m^3          # # at STP
density steel         = 0.283 lb/in^3         # # carbon steel
density aluminum      = 0.098 lb/in^3
density zinc          = 0.230 lb/in^3
density brass         = 0.310 lb/in^3         # # 80Cu-20Zn
density copper        = 0.295 lb/in^3
density iron          = 0.260 lb/in^3         # # cast iron
density nickel        = 0.308 lb/in^3
density tin           = 0.275 lb/in^3
density titanium      = 0.170 lb/in^3
density silver        = 0.379 lb/in^3
density nylon         = 0.045 lb/in^3
density polycarbonate = 0.045 lb/in^3


#
# energy units
#
[energy]
joule                = N*m
J                    = joule             # joule
kilojoule            = 1000 joule
kJ                   = kilojoule         # kilojoule
megajoule            = 1000 kilojoule
gigajoule            = 1000 megajoule
millijoule           = 0.001 joule
mJ                   = millijoule        # millijoule
calorie              = 4.1868 J
cal                  = calorie           # calorie
kilocalorie          = 1000 cal
kcal                 = kilocalorie       # kilocalorie
calorie food         = kilocalorie
Btu                  = cal*lb*R/g*K      # British thermal unit
British thermal unit = Btu
erg                  = cm*dyne
electronvolt         = 1.602176462e-19 J
eV                   = electronvolt      # electronvolt
kWh                  = kW*hour           # kilowatt-hour
kilowatt hour        = kWh
ton TNT              = 4.184e9 J


#
# power units
#
[power]
watt              = J/s
W                 = watt            # watt
kilowatt          = 1000 W
kW                = kilowatt        # kilowatt
megawatt          = 1000 kW
MW                = megawatt        # megawatt
gigawatt          = 1000 MW
GW                = gigawatt        # gigawatt
milliwatt         = 0.001 W
horsepower        = 550 ft*lbf/sec
hp                = horsepower      # horsepower
metric horsepower = 75 kgf*m/s


#
# frequency
#
[frequency]
hertz       = unit/sec
Hz          = hertz      # hertz
millihertz  = 0.001 Hz
kilohertz   = 1000 Hz
kHz         = kilohertz  # kilohertz
megahertz   = 1000 kHz
MHz         = megahertz  # megahertz
gigahertz   = 1000 MHz
GHz         = gigahertz  # gigahertz


#
# radioactivity
#
[radioactivity]
becquerel       = unit/sec
Bq              = becquerel     # becquerel
curie           = 3.7e10 Bq
millicurie      = 0.001 curie
roentgen        = 2.58e-4 coulomb/kg
[radiation dose]
gray            = J/kg
Gy              = gray          # gray
rad. abs. dose  = 0.001 Gy      # # commonly rad
sievert         = J/kg          # # equiv. dose
millisievert    = 0.001 sievert # # equiv. dose
Sv              = sievert       # sievert # equiv. dose
rem             = 0.01 Sv       # # roentgen equiv mammal
millirem        = 0.001 rem     # # roentgen equiv mammal


#
# viscosity
#
[dyn viscosity]
poise        = g/cm*s
P            = poise       # poise
centipoise   = 0.01 poise
cP           = centipoise  # centipoise

[kin viscosity]
stokes       = cm^2/s
St           = stokes      # stokes
centistokes  = 0.01 stokes
cSt          = centistokes # centistokes


#
# misc. units
#
[acceleration]
gravity                = 9.80665 m/s^2
[constant]
gravity constant       = 6.673e-11 N*m^2/kg^2
gas constant           = 8.314472 J/mol*K     # R
[fuel consumpt.]
mpg                    = mi/gal               # miles/gallon
"""



pronouns = """
he
she
it
him
her
his
hers
its
they
them
their
theirs
i
me
mine
we
us
our
ours
you
your
yours
one
one's
ones
anyone
anybody
anything
someone
somebody
something
everyone
everybody
everything
nothing
nobody
any
each
either
neither
all
most
some
several
none
both
few
many
himself
herself
itself
themselves
themself
myself
ourselves
ourself
oneself
this
that
these
those
who
whom
which
whose
whoever
whomever
whatever
whichever
what
"""
